
#include <CommandLineArgumentManager.h>
#include <sstream>
#include <algorithm>

#include "CommandLineArgumentParser.h"

using namespace CommandLineArgumentLib;
using namespace std;

CommandLineArgumentManager::CommandLineArgumentManager() : optionHelpMessageBuild(false)
{

}

CommandLineArgumentManager::CommandLineArgumentManager(const CommandLineArgumentManager& orig)
{
}

CommandLineArgumentManager& CommandLineArgumentManager::operator =(const CommandLineArgumentManager& orig)
{
}

CommandLineArgumentManager::~CommandLineArgumentManager()
{
}

void CommandLineArgumentManager::AddArgumentType(CLArgumentBase* argumentType)
{
    CLAOptionBase *option = dynamic_cast<CLAOptionBase*> (argumentType);
    if (option != NULL)
    {
        AddArgumentType(option);
        return;
    }

    CLARequiredValue *requiredValue = dynamic_cast<CLARequiredValue*> (argumentType);
    if (requiredValue != NULL)
    {
        AddArgumentType(requiredValue);
        return;
    }

    CLAOptionalValue *optionalValue = dynamic_cast<CLAOptionalValue*> (argumentType);
    if (optionalValue != NULL)
    {
        AddArgumentType(optionalValue);
        return;
    }

    throw CLAException("Argument type not supported by CommandLineArgumentManager.");
}

void CommandLineArgumentManager::AddArgumentType(CLAOptionBase* option)
{
    options.insert(make_pair(option->GetIdentifier(), option));
    optionList.push_back(option);
}

void CommandLineArgumentManager::AddArgumentType(CLARequiredValue* requiredValue)
{
    requiredValues.push_back(requiredValue);
}

void CommandLineArgumentManager::AddArgumentType(CLAOptionalValue* optionalValue)
{
    optionalValues.push_back(optionalValue);
}

CLAOptionBase* CommandLineArgumentManager::GetArgumentType(string optionIdentifier)
{
    map<string, CLAOptionBase*>::iterator iter;

    iter = options.find(optionIdentifier);

    if (iter == options.end())
    {
        stringstream ss;
        ss << optionIdentifier << " is no valid option";
        throw CLAException(ss.str());
    }

    return iter->second;
}

CLAValue* CommandLineArgumentManager::GetArgumentType(int index)
{
    if (index < requiredValues.size())
    {
        return requiredValues[index];
    }

    //TODO: Optional values

    throw CLAException("Index for value out of range");
}

bool CommandLineArgumentManager::AllRequiredValuesWereFound()
{
    vector<CLARequiredValue*>::iterator notFoundRequiredValueIterator;
    notFoundRequiredValueIterator = find_if(requiredValues.begin(), requiredValues.end(), not1(mem_fun(&CLARequiredValue::IsValueFound)));

    if (notFoundRequiredValueIterator != requiredValues.end())
        return false;

    return true;
}

string CommandLineArgumentManager::GetOptionHelpMessage() {
    if(optionHelpMessageBuild)
        return optionHelpMessage;

    AppendOptionsHelpMessage appendOptionObject;

    appendOptionObject = for_each(optionList.begin(), optionList.end(), appendOptionObject);

    optionHelpMessage = appendOptionObject.GetHelpMessage();
    optionHelpMessageBuild = true;
    return optionHelpMessage;
}