/*
 * File:   ParseValues_Test.cpp
 * Author: gruen
 *
 * Created on 30.12.2010, 09:53:08
 */

#include "ParseValues_Test.h"
#include <CLAParseArgumentFunctions.h>

using namespace CommandLineArgumentLib;

CPPUNIT_TEST_SUITE_REGISTRATION(ParseValues_Test);

ParseValues_Test::ParseValues_Test()
{
}

ParseValues_Test::~ParseValues_Test()
{
}

void ParseValues_Test::setUp()
{
    parser = new CommandLineArgumentParser();
}

void ParseValues_Test::tearDown()
{
    delete parser;
}

void ParseValues_Test::ParseRequiredValueString_Success()
{
    //arrange
    CLARequiredValue v;
    SingleValueSetting<std::string> s;
    int argc = 1;
    char *argv[1];

    s.ParseValue = &ParseArgumentFunctions::ParseString;
    v.SetSetting(&s);

    argv[0] = "aRequiredValue";

    parser->AddArgumentType(&v);

    //act
    parser->ParseArguments(argc, argv);

    //assert
    CPPUNIT_ASSERT(s.IsValid() == true);
    CPPUNIT_ASSERT(s.GetValue() == argv[0]);
}

void ParseValues_Test::ParseRequiredValueInt_Success()
{
    //arrange
    CLARequiredValue v;
    SingleValueSetting<int> s;
    int argc = 1;
    char *argv[1];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    v.SetSetting(&s);

    argv[0] = "123";

    parser->AddArgumentType(&v);

    //act
    parser->ParseArguments(argc, argv);

    //assert
    CPPUNIT_ASSERT(s.IsValid() == true);
    CPPUNIT_ASSERT(s.GetValue() == 123);
}

void ParseValues_Test::ParseTwoRequiredValuesStringInt_Success()
{
    //arrange
    CLARequiredValue v;
    SingleValueSetting<int> s;
    int argc = 1;
    char *argv[1];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    v.SetSetting(&s);

    argv[0] = "123";

    parser->AddArgumentType(&v);

    //act
    parser->ParseArguments(argc, argv);

    //assert
    CPPUNIT_ASSERT(s.IsValid() == true);
    CPPUNIT_ASSERT(s.GetValue() == 123);
}

void ParseValues_Test::ParseRequiredValueInt_ValueInvalid()
{
    //arrange
    bool error = false;
    CLARequiredValue v;
    SingleValueSetting<int> s;
    int argc = 1;
    char *argv[1];

    s.ParseValue = &ParseArgumentFunctions::ParseNumber;
    v.SetSetting(&s);

    argv[0] = "1a23";

    parser->AddArgumentType(&v);

    //act
    try
    {
        parser->ParseArguments(argc, argv);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}

void ParseValues_Test::ParseRequiredValueString_ValueMissing()
{
    //arrange
    bool error = false;
    CLARequiredValue v;
    SingleValueSetting<std::string> s;
    int argc = 0;
    char *argv[0];

    s.ParseValue = &ParseArgumentFunctions::ParseString;
    v.SetSetting(&s);

    parser->AddArgumentType(&v);

    //act
    try
    {
        parser->ParseArguments(argc, argv);
    }
    catch (CLAException e)
    {
        error = true;
    }

    //assert
    CPPUNIT_ASSERT(error == true);
}

