/* 
 * File:   CommandParser.cpp
 * Author: jochen
 * 
 * Created on 10. Januar 2011, 14:19
 */

using namespace std;

#include <CommandParser.h>
#include <iostream>
#include <utility>
#include <sys/time.h>
#include <typeinfo>

CommandParser::CommandParser() {
}

CommandParser::CommandParser(const CommandParser& orig) {
}

CommandParser::~CommandParser() {
}

void CommandParser::addCommand(InputCommandBase* command) {
    commandList.insert(make_pair(command->GetKey(), command));
}

void CommandParser::start() {
    struct timespec timespec;
    timespec.tv_sec = 0;
    timespec.tv_nsec = 1000000 * 40;

    struct timespec timespec2;
    timespec2.tv_sec = 0;
    timespec2.tv_nsec = 1000000 * 100;

    cerr << "Starting to read commands ... " << endl;
    while (true) {
        try {
            nanosleep(&timespec2, 0);
            controlCommunication->FlushOutputStream();
            controlCommunication->FillInputStream();
            string commandKey = ReadCommandKey();
            cerr << "Input for command " << commandKey << endl;
            if (commandKey == "") {
                cin.clear();
                nanosleep(&timespec, 0);
                continue;
            }

            if (commandKey == "help") {
                cerr << "Got help command key" << endl;
                ostream& stream = controlCommunication->GetOutputStream();
                stream << "Available commands: " << endl;
                stream << GetHelpMessage() << endl;
                continue;
            }

            InputCommandBase *command = FindInputCommand(commandKey);

            cerr << "Parsing Socket input " << endl;
            command->ReadParameters(controlCommunication->GetInputStream());
            cerr << "Executing command " << commandKey << endl;
            command->Execute();
        } catch (const char* c) {

            ostream& stream = controlCommunication->GetOutputStream(true);
            stream << c << endl;
            cerr << "Sync Input Buffer" << endl;
            (controlCommunication->GetInputStream()).sync();
        }
    }
}

string CommandParser::ReadCommandKey() {
    string commandKey;
    istream& stream = controlCommunication->GetInputStream();
    stream >> commandKey;
    return commandKey;
}

InputCommandBase* CommandParser::FindInputCommand(string key) {
    map<string, InputCommandBase*>::iterator commandIter;
    commandIter = commandList.find(key);

    if (commandIter != commandList.end())
        return (*commandIter).second;

    throw "Command not found";
}

string CommandParser::GetHelpMessage() {
    map<string, InputCommandBase*>::iterator commandIter;
    stringstream ss;

    for (commandIter = commandList.begin(); commandIter != commandList.end(); commandIter++) {
        ss << commandIter->first << ": " << commandIter->second->GetHelpMessage() << endl;
    }

    return ss.str();
}



