#include <UnixSocketControlCommunication.h>

#include <unistd.h>
#include <algorithm>
#include <errno.h>
#include <iostream>
#include <signal.h>

using namespace std;

UnixSocketControlCommunication *unixSocketControlCommunicationInstance = 0;

void RecreatePipe(int dummy) {
    cerr << "WARNING: SIGPIPE detected, trying recreation of unix socket ...";
    if (unixSocketControlCommunicationInstance) {
        unixSocketControlCommunicationInstance->ClearConnection(false);
        unixSocketControlCommunicationInstance->BindUnixSocket();
        cerr << "Unix socket recreated" << endl;
    } else {
        cerr << "Could not recreate unix socket, no UnixSocketControlCommunication instance" << endl;
    }
}

UnixSocketControlCommunication::UnixSocketControlCommunication() {
    ClearConnection();
    sockaddrLength = 0;

    unixSocketControlCommunicationInstance = this;
    signal(SIGPIPE, &RecreatePipe);
}

UnixSocketControlCommunication::~UnixSocketControlCommunication() {
    if (sockaddrLength > 0)
        unlink(sockaddr.sun_path);

    if (localSocketFd >= 0)
        close(localSocketFd);

    if (remoteSocketFd >= 0)
        close(remoteSocketFd);

    signal(SIGPIPE, SIG_DFL);
    unixSocketControlCommunicationInstance = 0;
}

UnixSocketControlCommunication::UnixSocketControlCommunication(const UnixSocketControlCommunication& orig) {
}

void UnixSocketControlCommunication::ClearConnection(bool closeConnection) {
    if(closeConnection && remoteSocketFd >= 0)
        close(remoteSocketFd);
    remoteSocketFd = -1;
}

void UnixSocketControlCommunication::CreateUnixSocket(string filePath) {
    ClearConnection();
    sockaddr.sun_family = AF_UNIX;
    strcpy(sockaddr.sun_path, filePath.c_str());

    localSocketFd = socket(AF_UNIX, SOCK_STREAM, 0);
    if (localSocketFd == -1)
        throw "Could not create socket";

    sockaddrLength = strlen(sockaddr.sun_path) + sizeof (sockaddr.sun_family);

    BindUnixSocket();
}

void UnixSocketControlCommunication::BindUnixSocket() {
    unlink(sockaddr.sun_path);

    // Bind socket
    int bound = bind(localSocketFd, (struct sockaddr *) &sockaddr, sockaddrLength);
    if (bound == -1) {
        stringstream ss;
        ss << "Could not connect socket (no. " << errno << ")";
        throw ss.str();
    }

    // start listening on socket
    int listening = listen(localSocketFd, 1);
    if (listening == -1) {
        stringstream ss;
        ss << "Could not start listening on socket (no. " << errno << ")";
        throw ss.str();
    }
}

void UnixSocketControlCommunication::AcceptConnectionRequest() {
    if (remoteSocketFd >= 0)
        return;

    remoteSocketFd = accept(localSocketFd, (struct sockaddr*) &sockaddr, (socklen_t*) & sockaddrLength);
    if (remoteSocketFd < 0) {
        stringstream ss;
        ss << "Accepting connection failed (no. " << errno << ")";
        throw ss.str();
    }
}

void UnixSocketControlCommunication::FillInputStream() {
    AcceptConnectionRequest();

    int maxLen = 256;
    char buffer[maxLen];

    for (int i = 0; i < maxLen; i++)
        buffer[i] = '\0';

    cerr << "Reset commandInput" << endl;
    commandInput.str("");
    commandInput.clear();
    int bytesRead = read(remoteSocketFd, buffer, maxLen);

    if (bytesRead > 0) {
        commandInput << buffer;
        cerr << "Read into input buffer:" << commandInput.str() << endl;
    } else {
        close(remoteSocketFd);
        remoteSocketFd = -1;
    }
}

void UnixSocketControlCommunication::FlushOutputStream() {
    int maxLen = 256;
    char buffer[maxLen];

    int length = commandOutput.str().size();

    stringstream tmpOutput;
    tmpOutput << commandOutput.rdbuf();
    length = tmpOutput.str().size();

    if(length == 0)
        return;

    int ret = write(remoteSocketFd, tmpOutput.str().c_str(), length);
    if (ret != length) {
      // solve warning.
    }

    cerr << "Wrote to output: " << tmpOutput.str() << endl;
}

