/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <SendSettings.h>
#include <PrrtParseFunctions.h>
#include <CLAParseArgumentFunctions.h>

using namespace std;
using namespace CommandLineArgumentLib;

SendSettings::SendSettings() {
    unicastAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    multicastAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    feedbackAddressSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    listenPortSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    bwestSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    minimumDistanceSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    rttSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    plrTargetSetting.ParseValue = &ParseArgumentFunctions::ParseFloat;
    qHoldbackTimeSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    qHoldbackMinSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    resendIntervalSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    useAhecSetting.ParseValue = &ParseArgumentFunctions::ParseBool;
    useFecSetting.ParseValue = &ParseArgumentFunctions::ParseBool;
    fixKSetting.ParseValue = &ParseArgumentFunctions::ParseBool;
    dataRateSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    ipPortSetting.ParseValue = &PrrtParseFunctions::ParseIpPort;
    adaptationSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    updateIntervalSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    trafficShapingSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;
    ahecCodingParametersSetting.ParseValue = &ParseArgumentFunctions::ParseString;
    timeoutSetting.ParseValue = &ParseArgumentFunctions::ParseNumber;

    multicastAddressSetting.SetValue("224.0.2.2");
    listenPortSetting.SetValue("5004");
    bwestSetting.SetValue("5000");
    minimumDistanceSetting.SetValue("10");
    rttSetting.SetValue("20");
    plrTargetSetting.SetValue("0.000001");
    qHoldbackTimeSetting.SetValue("300000");
    qHoldbackMinSetting.SetValue("18000");
    resendIntervalSetting.SetValue("10");
    useAhecSetting.SetValue("1");
    useFecSetting.SetValue("0");
    fixKSetting.SetValue("1");
    adaptationSetting.SetValue("0");
    updateIntervalSetting.SetValue("200");
    trafficShapingSetting.SetValue("4");
    ahecCodingParametersSetting.SetValue("200,2,20,40,0,1,0,1,2");
    timeoutSetting.SetValue("0");
}

SendSettings::SendSettings(const SendSettings& orig) {
}

SendSettings::~SendSettings() {
}

string SendSettings::ToString() const
{
    stringstream ss;

    ss << "Listen Port: " << listenPortSetting.GetValue() << endl;
    ss << "Bandwidth Estimation: " << bwestSetting.GetValue() << endl;
    ss << "Minimum distance: " << minimumDistanceSetting.GetValue() << endl;
    ss << "RTT: " << rttSetting.GetValue() << endl;
    ss << "AHEC Coding Parameters: " << ahecCodingParametersSetting.GetValue() << endl;
    ss << "PLR_target: " << plrTargetSetting.GetValue() << endl;
    ss << "QHoldbackTime: " << qHoldbackTimeSetting.GetValue() << endl;
    ss << "QHoldbackMin: " << qHoldbackMinSetting.GetValue() << endl;
    ss << "Resend Interval: " << resendIntervalSetting.GetValue() << endl;
    ss << "Use AHEC: " << useAhecSetting.GetValue() << endl;
    ss << "Use FEC: " << useFecSetting.GetValue() << endl;
    ss << "Fix K: " << fixKSetting.GetValue() << endl;
    ss << "Unicast Address: " << unicastAddressSetting.GetValue() << endl;
    ss << "Multicast Address: " << multicastAddressSetting.GetValue() << endl;
    ss << "Data Rate: " << dataRateSetting.GetValue() << endl;
    ss << "Clone IP: " << ipPortSetting.GetValue().first << endl;
    ss << "Clone Port: " << ipPortSetting.GetValue().second << endl;
    ss << "Adaptation Mode: " << adaptationSetting.GetValue() << endl;
    ss << "Update Interval of the Adaptation (ms): " << updateIntervalSetting.GetValue() << endl;
    ss << "Traffic Shaping with datarate (Mbps): " << trafficShapingSetting.GetValue() << endl;

    return ss.str();
}
