/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef SENDSETTINGS_H
#define	SENDSETTINGS_H

#include <string>
#include <typedefs.h>

#include <SettingBase.h>
#include <SingleValueSetting.h>

class SendSettings {
public:
    SendSettings();
    virtual ~SendSettings();

    unsigned short GetListenPort() const {
        return (unsigned short) listenPortSetting.GetValue();
    }

    int GetBWest() const {
        return bwestSetting.GetValue();
    }

    int GetRtt() const {
        return rttSetting.GetValue();
    }

    float GetPlrTarget() const {
        return plrTargetSetting.GetValue();
    }

    int GetMinimumDistance() const {
        return minimumDistanceSetting.GetValue();
    }

    int GetQHoldbackTime() const {
        return qHoldbackTimeSetting.GetValue();
    }

    int GetQHoldbackMin() const {
        return qHoldbackMinSetting.GetValue();
    }

    int GetResendInterval() const {
        return resendIntervalSetting.GetValue();
    }

    bool GetUseAhec() const {
        return useAhecSetting.GetValue();
    }

    bool GetUseFec() const {
        return useFecSetting.GetValue();
    }

    bool GetFixK() const {
        return fixKSetting.GetValue();
    }

    int GetAdaptation() const {
        return adaptationSetting.GetValue();
    }

    int GetUpdateInterval() const {
        return updateIntervalSetting.GetValue();
    }

    int GetTrafficShaping() const {
        return trafficShapingSetting.GetValue();
    }

    std::string GetAHECCodingParameters() const {
        return ahecCodingParametersSetting.GetValue();
    }

    bool IsUseMulticast() const {
        return !unicastAddressSetting.IsValid();
    }

    std::string GetTargetAddress() const {
        if (!IsUseMulticast())
            return unicastAddressSetting.GetValue();
        else
            return multicastAddressSetting.GetValue();
    }

    std::string GetFeedbackAddress() const {
        if(!IsUseMulticast() && !feedbackAddressSetting.IsValid())
            throw CommandLineArgumentLib::CLAException("Feedback address is not set.");

        if (!IsUseMulticast())
            return feedbackAddressSetting.GetValue();
        else
            return "0";
    }

    bool GetUseDummy() const {
        return dataRateSetting.IsValid();
    }

    int GetDataRate() const {
        return dataRateSetting.GetValue();
    }

    int GetTimeout() const {
        return timeoutSetting.GetValue();
    }

    bool GetUseCloning() const {
        return ipPortSetting.IsValid();
    }

    std::string GetCloneIP() const  {
        return ipPortSetting.GetValue().first;
    }

    unsigned int GetClonePort() const {
        return ipPortSetting.GetValue().second;
    }

    CommandLineArgumentLib::SettingBase* GetIpPortSetting() {
        return &ipPortSetting;
    }

    CommandLineArgumentLib::SettingBase* GetDataRateSetting() {
        return &dataRateSetting;
    }

    CommandLineArgumentLib::SettingBase* GetMulticastAddressSetting() {
        return &multicastAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUnicastAddressSetting() {
        return &unicastAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetFeedbackAddressSetting() {
        return &feedbackAddressSetting;
    }

    CommandLineArgumentLib::SettingBase* GetFixKSetting() {
        return &fixKSetting;
    }

    CommandLineArgumentLib::SettingBase* GetAdaptationSetting() {
        return &adaptationSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUpdateIntervalSetting() {
        return &updateIntervalSetting;
    }

    CommandLineArgumentLib::SettingBase* GetTrafficShapingSetting() {
        return &trafficShapingSetting;
    }

    CommandLineArgumentLib::SettingBase* GetAHECCodingParametersSetting() {
        return &ahecCodingParametersSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUseFecSetting() {
        return &useFecSetting;
    }

    CommandLineArgumentLib::SettingBase* GetUseAhecSetting() {
        return &useAhecSetting;
    }

    CommandLineArgumentLib::SettingBase* GetResendIntervalSetting() {
        return &resendIntervalSetting;
    }

    CommandLineArgumentLib::SettingBase* GetQHoldbackMinSetting() {
        return &qHoldbackMinSetting;
    }

    CommandLineArgumentLib::SettingBase* GetQHoldbackTimeSetting() {
        return &qHoldbackTimeSetting;
    }

    CommandLineArgumentLib::SettingBase* GetPlrTargetSetting() {
        return &plrTargetSetting;
    }

    CommandLineArgumentLib::SettingBase* GetRttSetting() {
        return &rttSetting;
    }

    CommandLineArgumentLib::SettingBase* GetMinimumDistanceSetting() {
        return &minimumDistanceSetting;
    }

    CommandLineArgumentLib::SettingBase* GetBWestSetting() {
        return &bwestSetting;
    }

    CommandLineArgumentLib::SettingBase* GetListenPortSetting() {
        return &listenPortSetting;
    }

    CommandLineArgumentLib::SettingBase* GetTimeoutSetting() {
        return &timeoutSetting;
    }

    std::string ToString() const;
private:
    SendSettings(const SendSettings& orig);

    IntSetting listenPortSetting;
    IntSetting bwestSetting;
    IntSetting minimumDistanceSetting;
    IntSetting rttSetting;
    FloatSetting plrTargetSetting;
    IntSetting qHoldbackTimeSetting;
    IntSetting qHoldbackMinSetting;
    IntSetting resendIntervalSetting;
    BoolSetting useAhecSetting;
    BoolSetting useFecSetting;
    BoolSetting fixKSetting;
    StringSetting unicastAddressSetting;
    StringSetting feedbackAddressSetting;
    StringSetting multicastAddressSetting;
    IntSetting dataRateSetting;
    StringUnsignedSetting ipPortSetting;
    IntSetting adaptationSetting;
    IntSetting updateIntervalSetting;
    IntSetting trafficShapingSetting;
    StringSetting ahecCodingParametersSetting;
    IntSetting timeoutSetting;

};

#endif	/* SENDSETTINGS_H */

