/* 
 * File:   SocketInitialiser.cpp
 * Author: jochen, manuel, shuai
 * 
 * Created on 11. Januar 2011, 16:28
 */

#include <SocketInitialiser.h>
#include <string>
#include <iostream>

using namespace std;
using namespace media_transport_protocol;

SocketInitialiser::SocketInitialiser() {
}

SocketInitialiser::SocketInitialiser(const SocketInitialiser& orig) {
}

SocketInitialiser::~SocketInitialiser() {
}

void SocketInitialiser::InitReceiverSocket(ReceiveSettings& settings, prrt_socket& prrtsocket) {
    string targetAddress(settings.GetTargetAddress());
    string senderAddress(settings.GetSenderAddress());

    InitReceiverSocket(settings, prrtsocket, senderAddress, targetAddress);
}

void SocketInitialiser::InitReceiverSocket(ReceiveSettings& settings, prrt_socket& prrtsocket, string senderIp, string receiverIp) {
    string targetAddress(receiverIp);
    string senderAddress(senderIp);

    std::cerr << "Starting Session " << targetAddress << " " << settings.GetPort() << " " << senderAddress << std::endl;

    option_TIMEOUT::option_value timeoutValue;
    if (settings.GetIsTimeoutActive()) {
        timeoutValue = option_TIMEOUT::StopAfterTimeout;
    } else {
        timeoutValue = option_TIMEOUT::ContinueAfterTimeout;
    }
    prrtsocket.setsockopt(option_TIMEOUT(timeoutValue));

    // The new bind() and connect().
    std::string as_recv( "as_recv" );
    std::string to_feedback( "to_feedback" );
    prrtsocket.bind(targetAddress, settings.GetPort(), as_recv);
    prrtsocket.connect(senderAddress, settings.GetPort() + 1, to_feedback);

    if (settings.GetBWESTMode() > 0 && settings.GetUpdateInterval() > 0) {
      prrtsocket.setsockopt( option_BWEST( static_cast<option_BWEST::Type>(settings.GetBWESTMode()), settings.GetUpdateInterval()) );
    }

    if (settings.GetIsFeedbackSuppressionActive() && settings.GetMaxFeedbackSentDelay() > 0) {
      prrtsocket.setsockopt( option_FBSUPP( option_FBSUPP::ON, settings.GetMaxFeedbackSentDelay() ) );
    }

    std::string str_delay_cc( settings.GetDelayCCParameters() );
    const char *delay_cc = str_delay_cc.c_str();
    double alpha = 0.0;
    double beta = 0.0;
    double gamma = 0.0;
    char *str_ptr_next = NULL;
    alpha = strtod(delay_cc, &str_ptr_next);
    beta = strtod(str_ptr_next, &str_ptr_next);
    gamma = strtod(str_ptr_next, NULL);
    prrtsocket.setDelayCCAlpha(alpha);
    prrtsocket.setDelayCCBeta(beta);
    prrtsocket.setDelayCCGamma(gamma);

    std::cerr << "Socket fd " << prrtsocket.getSockfd() << std::endl;
    //std::cerr << "delay CC alpha/beta/gamma " << prrtsocket.getDelayCCAlpha() << "/" << prrtsocket.getDelayCCBeta() << "/" << prrtsocket.getDelayCCGamma() << " string=" << delay_cc << std::endl;
}

void SocketInitialiser::InitSenderSocket(SendSettings& settings, prrt_socket& prrtsocket) {
    std::string s_streaming_addr = settings.GetTargetAddress();
    std::string s_feedback_addr = settings.GetFeedbackAddress();

    InitSenderSocket(settings, prrtsocket, s_feedback_addr, s_streaming_addr);
}

void SocketInitialiser::InitSenderSocket(SendSettings& settings, media_transport_protocol::prrt_socket& prrtsocket, std::string senderIp, std::string receiverIp){
    std::string s_streaming_addr = receiverIp;
    // HACK shine : here force to send feedback in the same multicast group if multicast sending data. I just matches the original intention. Because senderIp is always set to 0 in the case of multicast sending data.
    std::string s_feedback_addr = senderIp == "0" ? receiverIp : senderIp;
    //std::string s_feedback_addr = senderIp;
    // The new bind() and connect().
    std::string as_send( "as_send" );
    std::string to_receiver( "to_receiver" );
    //cerr << "SocketInitialiser::InitSenderSocket() bind s_feedback_addr=" << s_feedback_addr << " port=" << settings.GetListenPort() + 1 << " senderIp=" << senderIp << " receiverIp=" << receiverIp << endl;
    prrtsocket.bind(s_feedback_addr, settings.GetListenPort() + 1, as_send);
    //cerr << "SocketInitialiser::InitSenderSocket() connect s_streaming_addr=" << s_streaming_addr << " port=" << settings.GetListenPort() << endl;
    prrtsocket.connect(s_streaming_addr, settings.GetListenPort(), to_receiver);
    //prrtsocket.connect(s_streaming_addr, settings.GetListenPort(), s_feedback_addr);

    prrtsocket.set_param(param_BW_est(settings.GetBWest()));

    std::string ahec( settings.GetAHECCodingParameters() );
    std::vector<unsigned> v_ahec;
    const char *rr = ahec.c_str();
    stringstream ss;
    unsigned int num = 0;
    
    // Convert string to a unsigned int vector and cut off all unnecessary spaces in the string if the given AHEC is valid.
    unsigned int i = 0;
    while (i < ahec.size())
    {
      if (rr[i] == 44) {
        // The char is a comma.
        if (num >= 0) {
          v_ahec.push_back( num );
          if (v_ahec.size() == 1) {
            // The first number.
            ss << num;
          }
          else {
            ss << " " << num;
          }
        }
        else {
          cerr << "> SocketInitialiser initSenderSocket() ERROR > The given string of AHEC setting is invalid : " << (int) rr[i] << " in " << i << "-th " << ahec << endl;
        }
        num = 0;
      } // if (rr[i] == 44)
      else {
        // The char is not a comma.
        int digit = rr[i] - '0';
        if (-1 < digit && digit < 10 ) {
          // The char is a digit.
          num = num * 10 + digit;
        } // if-else : The char is a digit.
        else {
          cerr << "> SocketInitialiser initSenderSocket() ERROR > The given string of AHEC setting is invalid : " << (int) rr[i] << " in " << i << "-th " << ahec << endl;
        }
      } // if-else : the char is not a comma.
      
      i++;
    } // while (i < ahec.size())
    
    // Handle the last number.
    if (i == ahec.size() && num != 0) {
      v_ahec.push_back( num );
      if (v_ahec.size() == 1) {
        // The first number.
        ss << num;
      }
      else {
        ss << " " << num;
      }
    }
    
    if (v_ahec.size() < 5) {
      cerr << "SocketInitialiser initSenderSocket() ERROR > The given string of AHEC setting has not enough numbers : " << ahec << endl;
    }

    // Set the AHEC parameter of the PRRT socket.
    int d_target = v_ahec[0];
    int d_fec = v_ahec[1];
    int d_c = v_ahec[2];
    int d_req = v_ahec[3];
    int d_fb = v_ahec[4];
    int k = v_ahec[5];
    v_ahec.erase(v_ahec.begin(), v_ahec.begin() + 6);

    try {
      prrtsocket.set_param( param_AHEC( d_target, d_fec, d_c, d_req, d_fb, k, v_ahec ) );
    }
    catch ( prrt_error &e ) {
      cerr << "SocketInitialiser initSenderSocket() ERROR (ahec_param) > " << e.what() << endl;
    }
}
