/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef MAINCLASSPRRTCAT_H
#define	MAINCLASSPRRTCAT_H


#include <iostream>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <string>
#include <csignal>
#include <sys/time.h>
#include <prrt/prrt_sockets.h>
#include <prrt/prrt_params_base.h>
#include <CommandLineArgumentLib.h>
#include <SendSettings.h>
#include <SocketInitialiser.h>
#include <AdaptByLookup.hpp>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

using namespace std;
using namespace media_transport_protocol;
using namespace CommandLineArgumentLib;


class mainClassPrrtCat
{
    typedef void (*voidFunctionPointer)(int);
public:

    mainClassPrrtCat()
    : prrtsocket()
    , m_adapt()
    {
        this->forever = true;
        m_adaptation_mode = 0;
    }

    virtual int main(int argc, char *argv[])
    {
        CommandLineArgumentParser parser;
        SendSettings settings;

        InitCatParser(parser, settings);

        char* program = argv[0];

        unsigned char buf[buffersize];
        timeval tv_now, tv_last, tv_last_param_update;
        timeval tv_start;
        unsigned int lastGRTT = 50;
        double data_rate = 1000.0;  // in kbps.
        //unsigned short remainder = 0;
        double remainder = 0;
        gettimeofday(&tv_start, NULL);
        gettimeofday(&tv_last, NULL);
        long int timeout = 0;
        int n;
        static FILE* fd;

        fd = stdin;

        try
        {
            parser.ParseArguments(argc - 1, &argv[1]);

            if (parser.IsHelpFlagSet())
            {
                show_usage(program, parser.GetOptionHelpMessage());
                exit(1);
            }

            //prrt_socket prrtsocket;
            SocketInitialiser::InitSenderSocket(settings, prrtsocket);
            m_adaptation_mode = settings.GetAdaptation();
            
            // Enable adaptation.
            if (m_adaptation_mode == 1) {
              // Table lookup based adaptation.
              m_adapt.setSock( &prrtsocket );
              m_adapt.setUpdateInterval( settings.GetUpdateInterval() );
              m_adapt.start();
            }
            else if (m_adaptation_mode == 2) {
              // Online computing adaptation.
              prrtsocket.setsockopt( option_ADAPTATION( option_ADAPTATION::SIMPLE, settings.GetUpdateInterval() ) );
	            prrtsocket.startAdaptation();
            }

            data_rate = settings.GetDataRate();
            //data_rate = 1287 * 1.2;
            timeout = settings.GetTimeout();

            // UDP Support
            int udpsocket;
            const unsigned int udpsleep = 3000;
            struct sockaddr_in target_addr;
              
            std::string server_addr = settings.GetCloneIP();
            unsigned int server_port= settings.GetClonePort();

            bool useCloning = settings.GetUseCloning();

            if (useCloning)
            {
              printf("\n\nCloning active (%s:%d)!\n\n",server_addr.c_str(),server_port);
          
              if((udpsocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
                perror("socket issue");
                exit(1);
              }
              
              memset((char*) &target_addr,0,sizeof(target_addr));
              target_addr.sin_family = AF_INET;
              target_addr.sin_port = htons(server_port);
              
              if (inet_aton((const char *)server_addr.c_str(), &target_addr.sin_addr) == -1) {
                perror("inet_aton() issue");
                exit(1);
              }
            }			

            while (forever)
            {
                if (fd == stdin && !settings.GetUseDummy())
                {
                    // read from stdin
                    n = fread(buf, tsSize, tsNum, fd);
                    if (n != tsNum) cerr << "read" << "\t" << n << ", should be " << tsNum << endl;
                    if (n <= 0)
                    {
                        break;
                    }
                    sendpacket(buf, buffersize, prrtsocket);

                    if (useCloning) {
                      usleep(udpsleep);
                      sendto(udpsocket, buf, buffersize, 0, (const sockaddr *)&target_addr,sizeof(target_addr));
                    }
                }
                else
                {
                    // Dummy data source
                    gettimeofday(&tv_now, NULL);
                    timeval tv_diff;
                    timersub(&tv_now, &tv_last, &tv_diff);
                    gettimeofday(&tv_last, NULL);
                    //unsigned int t_diff = tv_diff.tv_sec * 1000 + tv_diff.tv_usec / 1000;
                    double t_diff = (double)tv_diff.tv_sec * 1000.0 + (double)tv_diff.tv_usec / 1000.0;
                    double data_vol = remainder + t_diff * data_rate / 8.0;
                    unsigned int num_packets = (unsigned int)floor(data_vol / (double)buffersize);
                    remainder = data_vol - (double)num_packets * (double)buffersize;

                    for (unsigned int p = 0; p < num_packets; p++) {
                        sendpacket(buf, buffersize, prrtsocket);

                        if (useCloning) {
                          usleep(udpsleep);
                          sendto(udpsocket, buf, buffersize, 0, (const sockaddr *)&target_addr,sizeof(target_addr));
                        }
                    }
                    usleep(1000);

                    timersub(&tv_now, &tv_last_param_update, &tv_diff);
		                if (!settings.GetDataRate()){
			                if ( tv_diff.tv_sec || tv_diff.tv_usec > lastGRTT*1000 ){
			                  prrt_socket::SocketStatistics sockStats;
			                  prrtsocket.getStats(&sockStats);
			                  gettimeofday(&tv_last_param_update, NULL);
			                  lastGRTT = sockStats.lastGRTT;
			                  //data_rate = sockStats.bwEst;
			                  data_rate = (double) prrtsocket.getStreamrate();
			                }
		                }
                }

                gettimeofday(&tv_now, NULL);
                timeval tv_diff;
                timersub(&tv_now, &tv_start, &tv_diff);
                if (0 < timeout && tv_diff.tv_sec > timeout) {
                  forever = false;
                }

            }


        }
        catch (prrt_error &me)
        {
            cerr << "ERROR: " << me.what() << endl;
        }
        catch (CLAException &parseException)
        {
            std::cerr << "Error when parsing arguments: " << parseException.what() << std::endl;
            show_usage(program, parser.GetOptionHelpMessage());
        }
        catch (exception &me)
        {
            std::cerr << "ERROR: " << me.what() << std::endl;
        }
        catch (const char* s) {
          std::cerr << "ERROR: " << s << std::endl;
        }

        fclose(fd);
        return 0;
    }

    void shutdown()
    {
      // stop the adaptation.
      if (m_adaptation_mode == 1) {
        m_adapt.stop();
        m_adaptation_mode = 0;
      }
      else if (m_adaptation_mode == 2) {
	      prrtsocket.stopAdaptation();
      }
      
      // Stop the while loop of sending operation.
        forever = false;
    }

protected:

    void InitCatParser(CommandLineArgumentParser &parser, SendSettings &settings)
    {

        stringstream helpMessage;
        helpMessage.precision(6);

        helpMessage.str("");
        helpMessage << "\n   Sender options:\n";
        helpMessage << "\t -sp <listen port at the receiver>  \t[default: " << settings.GetListenPort() << "]\n";
        listenPortOption.SetIdentifier("-sp");
        listenPortOption.SetHelpMessage(helpMessage.str());
        listenPortOption.SetSetting(settings.GetListenPortSetting());
        parser.AddArgumentType(&listenPortOption);

        helpMessage.str("");
        helpMessage << "\t -suc <unicast Address>            \tThe unicast address to send to\n";
        unicastAddressOption.SetIdentifier("-suc");
        unicastAddressOption.SetHelpMessage(helpMessage.str());
        unicastAddressOption.SetSetting(settings.GetUnicastAddressSetting());
        parser.AddArgumentType(&unicastAddressOption);

        helpMessage.str("");
        helpMessage << "\t -smc <multicast Address>            \tThe multicast Address to send to\n";
        multicastAddressOption.SetIdentifier("-smc");
        multicastAddressOption.SetHelpMessage(helpMessage.str());
        multicastAddressOption.SetSetting(settings.GetMulticastAddressSetting());
        parser.AddArgumentType(&multicastAddressOption);

        helpMessage.str("");
        helpMessage << "\t -fb <feedback Address>            \tThe feedback address for unicast transmissions\n";
        feedbackAddressOption.SetIdentifier("-fb");
        feedbackAddressOption.SetHelpMessage(helpMessage.str());
        feedbackAddressOption.SetSetting(settings.GetFeedbackAddressSetting());
        parser.AddArgumentType(&feedbackAddressOption);

        helpMessage.str("");
        helpMessage << "\t -code <AHEC Coding Parameters>        \t[default: " << settings.GetAHECCodingParameters() << "]\n";
        helpMessage << "\t\t\t\t\t\tExample : delay_target (200ms), delay_fec (2ms), decoding_timeout (20ms), feedback_timer (40ms), delay_feedback (0ms), k (1), n_p (0 1 2).\n";
        ahecCodingParametersOption.SetIdentifier("-code");
        ahecCodingParametersOption.SetHelpMessage(helpMessage.str());
        ahecCodingParametersOption.SetSetting(settings.GetAHECCodingParametersSetting());
        parser.AddArgumentType(&ahecCodingParametersOption);

        helpMessage.str("");
        helpMessage << "\t -bwe <tr>            \t\t\t[default: " << settings.GetBWest() << "]\n";
        bwestOption.SetIdentifier("-bwe");
        bwestOption.SetHelpMessage(helpMessage.str());
        bwestOption.SetSetting(settings.GetBWestSetting());
        parser.AddArgumentType(&bwestOption);

        helpMessage.str("");
        helpMessage << "\t -d <minimumDistance>            \t[default: " << settings.GetMinimumDistance() << "]\n";
        minimumDistanceOption.SetIdentifier("-d");
        minimumDistanceOption.SetHelpMessage(helpMessage.str());
        minimumDistanceOption.SetSetting(settings.GetMinimumDistanceSetting());
        parser.AddArgumentType(&minimumDistanceOption);

        helpMessage.str("");
        helpMessage << fixed << "\t -g <plrTarget>            \t\t[default: " << settings.GetPlrTarget() << "]\n";
        plrTargetOption.SetIdentifier("-g");
        plrTargetOption.SetHelpMessage(helpMessage.str());
        plrTargetOption.SetSetting(settings.GetPlrTargetSetting());
        parser.AddArgumentType(&plrTargetOption);

        helpMessage.str("");
        helpMessage << "\t -n <qHoldbackTime>            \t\t[default: " << settings.GetQHoldbackTime() << "]\n";
        qHoldbackTimeOption.SetIdentifier("-n");
        qHoldbackTimeOption.SetHelpMessage(helpMessage.str());
        qHoldbackTimeOption.SetSetting(settings.GetQHoldbackTimeSetting());
        parser.AddArgumentType(&qHoldbackTimeOption);

        helpMessage.str("");
        helpMessage << "\t -m <qHoldbackMin>            \t\t[default: " << settings.GetQHoldbackMin() << "]\n";
        qHoldbackMinOption.SetIdentifier("-m");
        qHoldbackMinOption.SetHelpMessage(helpMessage.str());
        qHoldbackMinOption.SetSetting(settings.GetQHoldbackMinSetting());
        parser.AddArgumentType(&qHoldbackMinOption);

        helpMessage.str("");
        helpMessage << "\t -w <resendInterval>            \t[default: " << settings.GetResendInterval() << "]\n";
        resendIntervalOption.SetIdentifier("-w");
        resendIntervalOption.SetHelpMessage(helpMessage.str());
        resendIntervalOption.SetSetting(settings.GetResendIntervalSetting());
        parser.AddArgumentType(&resendIntervalOption);

        helpMessage.str("");
        helpMessage << "\t -a <0|1>            \t\t\tUse AHEC [default: " << settings.GetUseAhec() << "]\n";
        useAhecOption.SetIdentifier("-a");
        useAhecOption.SetHelpMessage(helpMessage.str());
        useAhecOption.SetSetting(settings.GetUseAhecSetting());
        parser.AddArgumentType(&useAhecOption);

        helpMessage.str("");
        helpMessage << "\t -f <0|1>            \t\t\tUse FEC [default: " << settings.GetUseFec() << "]\n";
        useFecOption.SetIdentifier("-f");
        useFecOption.SetHelpMessage(helpMessage.str());
        useFecOption.SetSetting(settings.GetUseFecSetting());
        parser.AddArgumentType(&useFecOption);

        helpMessage.str("");
        helpMessage << "\t -y <0|1>            \t\t\tFix k [default: " << settings.GetFixK() << "]\n";
        fixKOption.SetIdentifier("-y");
        fixKOption.SetHelpMessage(helpMessage.str());
        fixKOption.SetSetting(settings.GetFixKSetting());
        parser.AddArgumentType(&fixKOption);

        helpMessage.str("");
        helpMessage << "\t -j <dataRate>            \t\tUses dummy data source with specified data rate (in KBit/s) instead of stdin (default)\n";
        dataRateOption.SetIdentifier("-j");
        dataRateOption.SetHelpMessage(helpMessage.str());
        dataRateOption.SetSetting(settings.GetDataRateSetting());
        parser.AddArgumentType(&dataRateOption);

        helpMessage.str("");
        helpMessage << "\t -c <$IP$_$Port$>            \t\tUses an additional (cloned) UDP connection to $IP$ on port $Port$\n";
        ipPortOption.SetIdentifier("-c");
        ipPortOption.SetHelpMessage(helpMessage.str());
        ipPortOption.SetSetting(settings.GetIpPortSetting());
        parser.AddArgumentType(&ipPortOption);

        helpMessage.str("");
        helpMessage << "\t -ad <0|1|2>            \t\tEnable adaptation [default: " << settings.GetAdaptation() << "]\n";
        helpMessage << "\t\t\t\t\t\t0: no adaptation; 1: table lookup based; 2: online computing.\n";
        adaptationOption.SetIdentifier("-ad");
        adaptationOption.SetHelpMessage(helpMessage.str());
        adaptationOption.SetSetting(settings.GetAdaptationSetting());
        parser.AddArgumentType(&adaptationOption);

        helpMessage.str("");
        helpMessage << "\t -i <updateInterval>    \t\tThe update interval of the adaptation in milliseconds [default: " << settings.GetUpdateInterval() << "]\n";
        updateIntervalOption.SetIdentifier("-i");
        updateIntervalOption.SetHelpMessage(helpMessage.str());
        updateIntervalOption.SetSetting(settings.GetUpdateIntervalSetting());
        parser.AddArgumentType(&updateIntervalOption);

        helpMessage.str("");
        helpMessage << "\t -ts <datarate>            \t\tEnable traffic shaping [default: " << settings.GetTrafficShaping() << "]\n";
        helpMessage << "\t\t\t\t\t\t0: traffic shaping with estimated bandwidth;\n";
        helpMessage << "\t\t\t\t\t\t>0: traffic shaping with the <datarate> in Mbps.\n";
        trafficShapingOption.SetIdentifier("-ts");
        trafficShapingOption.SetHelpMessage(helpMessage.str());
        trafficShapingOption.SetSetting(settings.GetTrafficShapingSetting());
        parser.AddArgumentType(&trafficShapingOption);

        helpMessage.str("");
        helpMessage << "\t -t <timeout>            \t\tProgram timeout in seconds [default: " << settings.GetTimeout() << "]\n";
        helpMessage << "\t\t\t\t\t\t0: the program won't terminate until the process is killed;\n";
        helpMessage << "\t\t\t\t\t\t>0: the program terminates after timeout seconds.\n";
        timeoutOption.SetIdentifier("-t");
        timeoutOption.SetHelpMessage(helpMessage.str());
        timeoutOption.SetSetting(settings.GetTimeoutSetting());
        parser.AddArgumentType(&timeoutOption);

        parser.AddHelpFlag("-h");
    }

private:

    void show_usage(char* program, string optionHelpMessage)
    {
        stringstream ss;
        ss << "\033[31mUsage: " << program << "[options] \033[0m\n\n";
        ss << "List of available options:\n";
        ss << optionHelpMessage;

        cout << ss.str();
    }

    void sendpacket(const unsigned char* buf, const uint& len, prrt_socket &prrtsocket)
    {
        prrtsocket.send((char*) buf, len, 0);
    }

    prrt_socket prrtsocket;
    bool forever;
    int m_adaptation_mode;    // indicates the adaptation mode.
    Adaptation::AdaptByLookup m_adapt;
    static const int buffersize = 1316;
    static const int tsSize = 188;
    static const int tsNum = 7;

    CLAOptionWithParameter listenPortOption;
    CLAOptionWithParameter bwestOption;
    CLAOptionWithParameter minimumDistanceOption;
    CLAOptionWithParameter plrTargetOption;
    CLAOptionWithParameter qHoldbackTimeOption;
    CLAOptionWithParameter qHoldbackMinOption;
    CLAOptionWithParameter resendIntervalOption;
    CLAOptionWithParameter useAhecOption;
    CLAOptionWithParameter useFecOption;
    CLAOptionWithParameter fixKOption;
    CLAOptionWithParameter unicastAddressOption;
    CLAOptionWithParameter feedbackAddressOption;
    CLAOptionWithParameter multicastAddressOption;
    CLAOptionWithParameter dataRateOption;
    CLAOptionWithParameter ipPortOption;

    CLAOptionWithParameter adaptationOption;
    CLAOptionWithParameter updateIntervalOption;
    CLAOptionWithParameter trafficShapingOption;
    CLAOptionWithParameter ahecCodingParametersOption;
    CLAOptionWithParameter timeoutOption;
};


#endif	/* MAINCLASSPRRTCAT_H */

