/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef MAINCLASSPRRTRELAY_H
#define	MAINCLASSPRRTRELAY_H

#include <iostream>
#include <signal.h>
#include <cstdlib>
#include <string.h>
#include <RelaySettings.h>
#include <prrt/prrt_relay.h>
#include <mainClassPrrtCat.h>
#include <mainClassPrrtRecv.h>

using namespace std;
using namespace media_transport_protocol;
using namespace CommandLineArgumentLib;

class mainClassPrrtRelay : protected mainClassPrrtRecv, protected mainClassPrrtCat {
public:

    mainClassPrrtRelay() {
        forever = true;
    }

    virtual int main(int argc, char** argv) {
        try {
            InitRelay(argc, argv);

            while (forever) {
                ;
            }

            cerr << "Relay stopped ... ending" << endl;

        } catch (prrt_error &me) {
            cerr << "Error in prrt lib (PrrtRelay): " << me.what() << endl;
        } catch (CLAException &parseException) {
            char* program = argv[0];
            show_usage(program, parser.GetOptionHelpMessage());
            cerr << "Error while parsing arguments: " << parseException.what() << endl;
        } catch (exception &e) {
            cerr << "Unexpected error: " << e.what() << endl;
        }

        return 0;
    }
    
    virtual void shutdown() {
        forever = false;
        relay->Stop();
        delete prrtsocketSend;
        delete prrtsocketRecv;
        delete relay;
    }

protected:  

    int InitRelay(int argc, char** argv) {
        InitCatParser(parser, sendSettings);
        InitRecvParser(parser, recvSettings);
        InitRelayParser(parser, relaySettings);

        char* program = argv[0];

        parser.ParseArguments(argc - 1, &argv[1]);

        if (parser.IsHelpFlagSet()) {
            show_usage(program, parser.GetOptionHelpMessage());
            exit(1);
        }

        //            cerr << recvSettings.ToString();
        //            cerr << sendSettings.ToString();

        relay = new prrt_relay();
        prrtsocketRecv = new prrt_socket();
        prrtsocketSend = new prrt_socket();

        SocketInitialiser::InitReceiverSocket(recvSettings, *prrtsocketRecv);
        SocketInitialiser::InitSenderSocket(sendSettings, *prrtsocketSend);

        relay->SetReceivingSocket(prrtsocketRecv);
        relay->SetSendingSocket(prrtsocketSend);
        relay->SetRelayName(relaySettings.GetRelayName());
        relay->Start();

        return 0;
    }

private:

    void InitRelayParser(CommandLineArgumentParser &parser, RelaySettings &settings){
        relayNameValue.SetSetting(settings.GetRelayNameSetting());
        parser.AddArgumentType(&relayNameValue);
    }

    virtual void show_usage(char* program, std::string optionMessage) {
        std::stringstream ss;
        ss << "\033[31mUsage: " << program << " [options] <sender> <relayname>\033[0m\n\n";
        ss << "List of available options:\n" << optionMessage;

        cerr << ss.str();
    }
    
    bool forever;

protected:
    CommandLineArgumentParser parser;
    CLARequiredValue relayNameValue;

    prrt_relay *relay;
    ReceiveSettings recvSettings;
    SendSettings sendSettings;
    RelaySettings relaySettings;
    prrt_socket *prrtsocketRecv;
    prrt_socket *prrtsocketSend;
};

#endif	/* MAINCLASSPRRTRELAY_H */

