#ifndef _PCR_H_
#define _PCR_H_

unsigned short tsp_hasAdaptationField ( const unsigned char *tsHeader )/*{{{*/
{
    return ( tsHeader[3] & 0x20 );
}/*}}}*/

unsigned short tsp_hasAdaptationFieldAndPayload ( const unsigned char *tsHeader )/*{{{*/
{
    return ( tsHeader[3] & 0x30 );
}/*}}}*/

unsigned short tsp_isPayloadUnitStart ( const unsigned char *tsHeader )/*{{{*/
{
    return ( tsHeader[1] & 0x40 );
}/*}}}*/

unsigned short tsp_AdaptationFieldLength ( const unsigned char *tsHeader )/*{{{*/
{
    return tsp_hasAdaptationField(tsHeader) ? tsHeader[4]+1 : 0;
}/*}}}*/

unsigned short tsp_isStartofIframe ( const unsigned char *tsHeader )/*{{{*/
{
	if(!tsp_hasAdaptationFieldAndPayload(tsHeader)) return 0;
	if(!tsp_isPayloadUnitStart(tsHeader)) return 0;
	unsigned int offset = 4;
    offset += tsp_AdaptationFieldLength(tsHeader);
	return ( tsHeader[offset+1] & 0x8 );
}/*}}}*/

unsigned short tsp_setDiscontinuity ( unsigned char *tsHeader )/*{{{*/
{
	if(!tsp_hasAdaptationFieldAndPayload(tsHeader)) return 0;
	if(!tsp_isPayloadUnitStart(tsHeader)) return 0;
	unsigned int offset = 4;
	tsHeader[offset+1] = tsHeader[offset+1] | 0x80;
	return 1;
}/*}}}*/

unsigned short tsp_hasPCR ( const unsigned char *tsHeader )/*{{{*/
{
    return ( tsp_hasAdaptationField ( tsHeader ) && ( ( tsHeader[5] >> 4 ) & 0x01 ) );
}/*}}}*/

bool discontinuityIndicator ( const unsigned char *tsPacket )/*{{{*/
{
    if ( tsPacket[5] >> 7 )
        std::cerr << "PCR discontinuity" << std::endl;
    return ( tsPacket[5] >> 7 );
}/*}}}*/

unsigned long long tsPCR27M ( const unsigned char *tsPacket )/*{{{*/
{
    unsigned long long pcr = ( tsPacket[6] & 0xFF ); 
    for ( unsigned char i = 0; i < 4; i++ ){
        pcr = pcr << 8;
        pcr |= ( tsPacket[7+i] & 0xFF );
    }
    pcr = pcr << 2;
    pcr |= ( ( tsPacket[11] & 0xC0 ) >> 6 );
    pcr = (pcr >> 9)*300 + pcr%300;
    return pcr;
}/*}}}*/

unsigned long long tsPCR90K ( const unsigned char *tsPacket )/*{{{*/
{
    unsigned long long pcr = (unsigned int)( tsPacket[6] ) << 25 | (unsigned int)( tsPacket[7] ) << 17 | (unsigned int)( tsPacket[8] ) << 9 | (unsigned int)( tsPacket[9] ) << 1 | (unsigned int)( tsPacket[10] ) >> 7; 
    return pcr;
}/*}}}*/

#endif
