/* (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "analysis.h"

#include <sstream>
#include <vector>
#include <iostream>

using namespace media_transport_protocol;
using namespace std;

// NOTE : Please make them consistent with the definition in prrt_module/prrt_packet.h
static const int IP_HEADER_SIZE = 20;
static const int UDP_HEADER_SIZE = 8;
static const int PRRT_HEADER_SIZE = 4 + UDP_HEADER_SIZE;
static const int PRRT_DATA_HEADER_SIZE = 12 + PRRT_HEADER_SIZE;
static const int PRRT_RED_HEADER_SIZE = 4 + PRRT_HEADER_SIZE;
static const int MAX_DATA_SIZE = 1500; /*maximum size of incoming packet data in bytes*/
static const int MAX_PRRT_PACKET_SIZE = PRRT_DATA_HEADER_SIZE + MAX_DATA_SIZE;

prrt_analysis::prrt_analysis() {

	m_plr = 0;

       /* ----populate values of slope and intercept in map rho_0 */  

       rho_0.insert (pair<double, LineParam*> (0.001, new LineParam(1.00671140939597,1.99328859060403)));
       rho_0.insert (pair<double, LineParam*> (0.021, new LineParam(1.06711409395973,2.93288590604027)));
       rho_0.insert (pair<double, LineParam*> (0.041, new LineParam(1.10738255033557,3.89261744966443)));
       rho_0.insert (pair<double, LineParam*> (0.061, new LineParam(1.15436241610738,3.84563758389262)));
       rho_0.insert (pair<double, LineParam*> (0.081, new LineParam(1.18791946308725,4.81208053691275)));
       rho_0.insert (pair<double, LineParam*> (0.101, new LineParam(1.22818791946309,5.77181208053691)));
       rho_0.insert (pair<double, LineParam*> (0.121, new LineParam(1.26845637583893,5.73154362416107)));
       rho_0.insert (pair<double, LineParam*> (0.141, new LineParam(1.30872483221477,6.69127516778524)));
       rho_0.insert (pair<double, LineParam*> (0.161, new LineParam(1.34899328859060,6.65100671140940)));
       rho_0.insert (pair<double, LineParam*> (0.181, new LineParam(1.38926174496644,7.61073825503356)));
       rho_0.insert (pair<double, LineParam*> (0.201, new LineParam(1.44295302013423,7.55704697986577)));
       rho_0.insert (pair<double, LineParam*> (0.221, new LineParam(1.48322147651007,8.51677852348993)));
       rho_0.insert (pair<double, LineParam*> (0.241, new LineParam(1.53691275167785,8.46308724832215)));
       rho_0.insert (pair<double, LineParam*> (0.261, new LineParam(1.58389261744966,9.41610738255034)));
       rho_0.insert (pair<double, LineParam*> (0.281, new LineParam(1.63758389261745,9.36241610738255)));
      
       /* ----populate values of slope and intercept in map rho_0_05 */  

       rho_0_05.insert (pair<double, LineParam*> (0.001, new LineParam(1.00671140939597,2.99328859060403)));
       rho_0_05.insert (pair<double, LineParam*> (0.021, new LineParam(1.07382550335570,3.92617449664430)));
       rho_0_05.insert (pair<double, LineParam*> (0.041, new LineParam(1.11409395973154,4.88590604026846)));
       rho_0_05.insert (pair<double, LineParam*> (0.061, new LineParam(1.15436241610738,4.84563758389262)));
       rho_0_05.insert (pair<double, LineParam*> (0.081, new LineParam(1.19463087248322,5.80536912751678)));
       rho_0_05.insert (pair<double, LineParam*> (0.101, new LineParam(1.23489932885906,5.76510067114094)));
       rho_0_05.insert (pair<double, LineParam*> (0.121, new LineParam(1.27516778523490,6.72483221476510)));
       rho_0_05.insert (pair<double, LineParam*> (0.141, new LineParam(1.31543624161074,7.68456375838926)));
       rho_0_05.insert (pair<double, LineParam*> (0.161, new LineParam(1.36241610738255,7.63758389261745)));
       rho_0_05.insert (pair<double, LineParam*> (0.181, new LineParam(1.40268456375839,8.59731543624161)));
       rho_0_05.insert (pair<double, LineParam*> (0.201, new LineParam(1.44966442953020,8.55033557046980)));
       rho_0_05.insert (pair<double, LineParam*> (0.221, new LineParam(1.49664429530201,9.50335570469799)));
       rho_0_05.insert (pair<double, LineParam*> (0.241, new LineParam(1.55033557046980,9.44966442953020)));
       rho_0_05.insert (pair<double, LineParam*> (0.261, new LineParam(1.59731543624161,10.4026845637584)));
       rho_0_05.insert (pair<double, LineParam*> (0.281, new LineParam(1.63087248322148,10.3691275167785)));
       
       /* ----populate values of slope and intercept in map rho_0_1 */  

       rho_0_1.insert (pair<double, LineParam*> (0.001, new LineParam(1.00671140939597,3.99328859060403)));
       rho_0_1.insert (pair<double, LineParam*> (0.021, new LineParam(1.07382550335570,4.92617449664430)));
       rho_0_1.insert (pair<double, LineParam*> (0.041, new LineParam(1.11409395973154,5.88590604026846)));
       rho_0_1.insert (pair<double, LineParam*> (0.061, new LineParam(1.16107382550336,5.83892617449664)));
       rho_0_1.insert (pair<double, LineParam*> (0.081, new LineParam(1.20134228187919,6.79865771812081)));
       rho_0_1.insert (pair<double, LineParam*> (0.101, new LineParam(1.24161073825503,6.75838926174497)));
       rho_0_1.insert (pair<double, LineParam*> (0.121, new LineParam(1.28187919463087,7.71812080536913)));
       rho_0_1.insert (pair<double, LineParam*> (0.141, new LineParam(1.32885906040268,7.67114093959732)));
       rho_0_1.insert (pair<double, LineParam*> (0.161, new LineParam(1.36912751677852,8.63087248322148)));
       rho_0_1.insert (pair<double, LineParam*> (0.181, new LineParam(1.40939597315436,9.59060402684564)));
       rho_0_1.insert (pair<double, LineParam*> (0.201, new LineParam(1.46308724832215,9.53691275167785)));
       rho_0_1.insert (pair<double, LineParam*> (0.221, new LineParam(1.50335570469799,10.4966442953020)));
       rho_0_1.insert (pair<double, LineParam*> (0.241, new LineParam(1.56375838926175,10.4362416107383)));
       rho_0_1.insert (pair<double, LineParam*> (0.261, new LineParam(1.61073825503356,11.3892617449664)));
       rho_0_1.insert (pair<double, LineParam*> (0.281, new LineParam(1.61744966442953,12.3825503355705)));

       /*********************************************************/

	   initialize(Factorials, MAX_N, -1, sizeof(long double));
}

prrt_analysis::~prrt_analysis() {
	// Release the dynamic objects of the three maps
	std::map<double, LineParam*>::iterator iter;

  for (iter = rho_0.begin(); rho_0.end() != iter; iter++) {
    delete iter->second;
  }
  for (iter = rho_0_05.begin(); rho_0_05.end() != iter; iter++) {
    delete iter->second;
  }
  for (iter = rho_0_1.begin(); rho_0_1.end() != iter; iter++) {
    delete iter->second;
  }
}


/*----------Compute factorial of a number x----------*/

long double prrt_analysis::factorial(int x){
	if (Factorials[x] < 0){
		if (x <= 1){
			//printf("Factorials[%u] (branch x<=1) = %f\n", x, (float)1);
			Factorials[x] = 1;
			return 1;
		}
		else{
			Factorials[x] = (long double)x * factorial(x-1);
			//printf("Factorials[%u] (else) = %f\n", x, (float) Factorials[x]);
			return Factorials[x];
		}
	}
	else {
		//printf("Factorials[%u] (else 2) = %f\n", x, (float) Factorials[x]);
		return Factorials[x];
	}
}


//Calculate Pd(n,k,i,j) Using  Fast Factorial Technique

long double prrt_analysis::fastfactorial(unsigned int *num, unsigned int NUM_SIZE, unsigned int *denom, unsigned int DENOM_SIZE){	
	
 	long double new_num[NUM_SIZE];
 	long double new_denom[DENOM_SIZE];
 	long double Pd;
 	int q, p, t;
    
//sort the arrays num and denom.This Sorts them in Ascendig order!!NOT GOOD.Careful.
 	//for(q=0; q<NUM_SIZE; q++){
	//	std::cerr << "num[" << q << "] " << num[q] << " denom[" << q << "] " << denom[q] << std::endl;
	//}
	//for( ; q<DENOM_SIZE; q++){	
	//	std::cerr << "denom[" << q << "] " << denom[q] << std::endl;
	//	}
	//bubblesort(num, NUM_SIZE);
 	//bubblesort(denom, DENOM_SIZE);
 	//for(q=0; q<NUM_SIZE; q++){
	//	std::cerr << "sortednum[" << q << "] " << num[q] << " sorteddenom[" << q << "] " << denom[q] << std::endl;
	//}
	//for( ; q<DENOM_SIZE; q++){	
	//	std::cerr << "sorteddenom[" << q << "] " << denom[q] << std::endl;
	//	}
    
//Initialize new_num and new_denom to 1
	initialize(new_num, NUM_SIZE, 1, sizeof(long double));
	initialize(new_denom, DENOM_SIZE, 1, sizeof(long double));

 	for(q=NUM_SIZE-1, t=DENOM_SIZE-1; q>=0; q--, t--){
		if(denom[t]<num[q]){
			if(denom[t]==0){
         			new_num[q]=factorial(num[q]);
			}
      			else{
				new_num[q]=denom[t]+1;
            			for(p=denom[t]+2; p<=num[q]; p++)
					new_num[q]=new_num[q]*p;
         		}		
     		}
    		else{
			if(num[q]<denom[t]){
				if(num[q]==0){
					new_denom[t]=factorial(denom[t]);
				}
        			else{
					new_denom[t]=num[q]+1;
          				for(p=num[q]+2; p<=denom[t]; p++)
             					new_denom[t]=new_denom[t]*p;
         			}
       			}
     		}
        
   	}
	if(DENOM_SIZE>NUM_SIZE){
		
		for(t=DENOM_SIZE-NUM_SIZE-1; t>=0; t--)
			new_denom[t]=factorial(denom[t]);		
		}
 	//new_denom[0]=factorial(denom[0]);
 
//calculate Pd
 	Pd=1.0;
 	for(q=0; q<NUM_SIZE; q++){
   		Pd=Pd * (new_num[q]/new_denom[q]);
		//std::cerr << "Pd " << Pd << " new_num[" << q << "] " << new_num[q] << " new_denom[" << q << "] " << new_denom[q] << std::endl;
	}
	for( ; q<DENOM_SIZE; q++){	
 		Pd=Pd/new_denom[q];
		//std::cerr << "Pd " << Pd << " new_denom[" << q << "] " << new_denom[q] << std::endl;
		}
 	return Pd;
}

//Bubble Sort.This function is used to sort an array

void prrt_analysis::bubblesort(unsigned int *tosort, unsigned int length) {
	long double tmp;
	int i,j;
	for(i = 0;i<length;i++) {
		for(j = length - 1;j>=i+1;j--) {
			if (tosort[j] > tosort[j-1]) {
				tmp = tosort[j];
				tosort[j] = tosort[j-1];
				tosort[j-1] = tmp;
			}
		}
	}
}


//Initialize new_num and new_denom arrays to ones.Important step.

void prrt_analysis::initialize(void *array, unsigned int arraylen, long double value, unsigned int sizeType){ 
  	unsigned int q;
  	unsigned int *pint;
  	long double *pldouble;
  	if ( sizeType == sizeof(int) ){
      		pint=(unsigned int*)array;
      	for(q=0; q<arraylen; q++)
           	pint[q]=value;
  	} 
  	else if (sizeType == sizeof(long double) ){ 
      		pldouble=(long double*)array;
      	for(q=0; q<arraylen; q++)
        	pldouble[q]=value;
  	}
}

/*------Initialize 2D array to a value------*/ 

void prrt_analysis::initialize2D (long double *array, long double value, unsigned int row, unsigned int col){
 	unsigned int q;
	for(q=0; q<row*col; q++)
    		*(array + q) = value;
}


//Calculate PLR Performance 
long double prrt_analysis::PLRperformance(unsigned int n, unsigned int k, float Pe, unsigned int *Np, unsigned int len, float Pf, unsigned int R, float rho){

  	long double Pd[k+1][n+1];   //initialize to =0
  	initialize2D((long double *)Pd,0,k+1,n+1);

  	unsigned int  q, p;
	unsigned int i, j, nr, nw;
  	long double Ei = 0 , PLR, Pn;
	unsigned int NUM_SIZE, DENOM_SIZE;

	NUM_SIZE=4;
	DENOM_SIZE=5;

	unsigned int num[NUM_SIZE];
	unsigned int denom[DENOM_SIZE];

	for(i=1; i<=k; i++){ 		
		for(j=((n-k+1 >=i)? n-k+1 :i ); j<=n-k+i; j++){
			//std::cerr << "n " << n << " k " << k << " i " << i << " j " << j << std::endl;
			fill_num_denom(num, NUM_SIZE, denom, DENOM_SIZE, n, k, i, j);
			//std::cerr << "Pdist " << Pdist(j, n, Pe, rho) << std::endl;
			Ei=Ei + i * fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE) * Pdist(j, n, Pe, rho);
			//std::cerr << "Ei " << Ei << std::endl;
		}
	}

 	PLR=Ei/(long double)k;
	return PLR;

  	//long double Pd[k+1][n+1];   //initialize to =0
  	//initialize2D((long double *)Pd,0,k+1,n+1);

  	//unsigned int  q, p;
	//unsigned int i, j, nr, nw;
  	//long double Ei = 0 , PLR, Pn;
	//unsigned int NUM_SIZE, DENOM_SIZE;

	//NUM_SIZE=4;
	//DENOM_SIZE=5;

	//unsigned int num[NUM_SIZE];
	//unsigned int denom[DENOM_SIZE];

	//FBdistribution(Np, len, Pf, R);

	//for(nr=0; nr<len; nr++){
	//	nw=k+Np[0]+ FBdist[nr][0];
	//	Pn=FBdist[nr][1];
	//	std::cerr << "Pn " << Pn << std::endl;
	//	for(i=1; i<=k; i++){ 		
	//		for(j=((nw-k+1 >=i)? nw-k+1 :i ); j<=nw-k+i; j++){
	//			fill_num_denom(num, NUM_SIZE, denom, DENOM_SIZE, nw, k, i, j);
	//			Ei=Ei + i * Pn * fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE) * Pdist(j, nw, Pe, rho);
	//			std::cerr << "Ei " << Ei << std::endl;
	//		}
	//	}
	//}
 	//PLR=Ei/(long double)k;
	//return PLR;
}

//Calculate Hypergeometric Distribution Pdist
long double prrt_analysis::Pdist(unsigned int d, unsigned int n, float Pe, const float rho){
	
	unsigned int NUM_SIZE = 4, DENOM_SIZE = 5;
	unsigned int num[NUM_SIZE];
	unsigned int denom[DENOM_SIZE];

	float alpha, beta;
	long double PGE[MAX_N];
	unsigned int i;
	long double f;

	//rho = 0.0;
	alpha = rho + Pe*(1-rho);
	beta = 1 - Pe*(1-rho);	

 /* Dist is a symmetric matrix and if d<=n set the value to 0 */
	//printf("Requesting Pdist: d = %u, n = %u\n", d, n);
   		if(d<=n){
   			if(Dist[d][n]==-1){
				//if(rho==0.0){
				if(1){
					fill_num_denom(num, NUM_SIZE, denom, DENOM_SIZE, n, d, 0, n-d);
					f = fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE);
					//printf("fastfactorial=%f \n",(float)f);
      					Dist[d][n]=powl(Pe, d)*powl(1.0-Pe, n-d)/f;
					} 	

				else{
					GEdistribution(alpha, beta, PGE, n);
					for(i=0; i<=n; i++)
						Dist[i][n]=PGE[i];
					}
			}
  		}
 		else 
  			Dist[d][n]=0; 
		
 	return Dist[d][n];
}

//Calculate Redundancy Performance

long double prrt_analysis::RIperformance(const unsigned int R, const unsigned int n, const unsigned int k, const unsigned int *Np, const unsigned int r, const float Pe, const float rho){      
// r=length or dimension of Np
/* R=Number of Receivers */
 	long double RI, Pr;
 	unsigned int w, q, nw;
	RI=Np[0];
 	nw=k+Np[0];
 	for(w=1; w<r; w++){
     		Pr=0;
     		for(q=nw-k+1; q<=nw; q++)
        		Pr=Pr+Pdist(q, nw, Pe, rho);
     		nw=nw+Np[w];
     		RI=RI + (1-powl(1-Pr, R)) * Np[w];
   	}
 	RI=RI/(long double)k;
 //printf("RI=%Lf K=%d \n",RI,k);
 	return RI;
}

double prrt_analysis::optimizeHEC(float Dtarget, float Ptarget, float CBw, float SBw, unsigned int R, float Pe, float RTT, float Ts, float Ts_min, float Pf, float rho, unsigned int LP, float Dhost, float DTx, float DPL, float DFB, float DSup){
  	float Dactual;   //Actual coding delay
	float Dresp;
	float D_FEC;
	float D_C;
	float D_REQ;
  	unsigned int m, Nrr, tNrr, sum;
  	unsigned int k, k_opt;
  	unsigned int n;
  	unsigned int t;
  	unsigned int v;
  	unsigned int Nrr_max, k_max, len_opt=1;
  	double PLRres=1.0, tPLRres=1.0, RIres=1.0, tRIres=1.0;
  
	Dresp = DSup + RTT + Dhost + DTx;
 	m=(unsigned int) fminf(60.0, floorf(Dtarget/Ts_min));
  	Nrr_max=floor((Dtarget-Ts-DTx-(Dhost+RTT)/2-DFB)/Dresp);
  	unsigned int Np[Nrr_max+1];           
  	unsigned int Np_opt[Nrr_max+1];
         
	k_opt = 1; 
  	initialize(Np_opt, Nrr_max+1, 0, sizeof(int));

	//Use only one of the following lines at a time. Direct computation or linear approximation

	//Uncomment the following line for direct computation of Optimum values

	//std::cerr << "dtarget " << Dtarget << " DTx " << DTx << " Dhost " << Dhost << " Dresp " << Dresp << " DPL " << DPL << " Ts " << Ts << " RTT " << RTT << " m " << m << " Nrr_max " << Nrr_max << " len_opt " << len_opt << " k_opt " << k_opt << " Np_opt " << Np_opt << " PLRres " << PLRres << " RIres " << RIres << " Ptarget " << Ptarget << " Pe " << Pe << " Pf " << Pf << " R " << R << " rho " << rho << std::endl;

	directOptimumComputation(Dtarget, DTx, Dhost, Dresp, DPL, Ts, RTT, Nrr_max, m, len_opt, k_opt, Np_opt, PLRres, RIres, Ptarget, Pe, Pf, R, rho);
	
	//Uncomment the following line for starting with linear approximation for Optimum values computation

	//linearApproximateOptimumComputation(Dtarget,RTT,Ts, m, len_opt, k_opt, Np_opt, PLRres, RIres, Ptarget, Pe, Pf, R, rho);
	
	D_C = (float)k_opt * Ts;
	D_FEC = D_C + DTx + 0.5*(RTT+Dhost) + DPL;
	vector<unsigned> n_cc_opt;
	D_REQ = Dresp;

	Dactual = (float)k_opt * Ts + DTx + 0.5*(RTT+Dhost) + DPL + DFB + (len_opt-1) * D_REQ;

 	//printf(" k_opt=%d \n",k_opt);
 	//
	////Print out the Optimal Parameter set
	//
 	//printf(" D_target= %f \n Delay_Feedback= %f \n PLR_res= %.10e \n RIres= %.6e \n Dactual Coding Delay= %f \n D_FEC= %f \n",
	//	Dtarget, DFB, PLRres, RIres, Dactual, k_opt*Ts);
	//
	//
 	//printf(" Np_opt=[");
	//for (t=0; t < len_opt; t++){
    	//	printf("%d ",Np_opt[t]);
	//	}	
 	//printf("] \n");

	A_HEC.set_d_fec((unsigned int)D_FEC);
	A_HEC.set_d_c((unsigned int)D_C);
	A_HEC.set_d_req((unsigned int)D_REQ);
	A_HEC.set_d_fb((unsigned int)DFB);
	//std::cerr << "RTT " << RTT << ", DTx " << DTx << ", D_C " << D_C << ", D_FEC " << D_FEC << ", A_HEC.D_REQ " << A_HEC.get_d_req() << ", D_actual " << Dactual << ", k " << k_opt;
 	for (t=0; t < len_opt; t++){
			n_cc_opt.push_back(Np_opt[t]);
			//std::cerr << ", Np[" << t << "] " << Np_opt[t] ;
	}
	//std::cerr << std::endl;
		
	A_HEC.set_n_p(n_cc_opt);
	A_HEC.set_k(k_opt);
	A_HEC.set_d_target((unsigned int)Dtarget);

  return RIres;
}

//Compute Optimum values starting with Linear Approximation

void prrt_analysis::linearApproximateOptimumComputation(unsigned int Dtarget,unsigned int RTT, float Ts, int m, int& len_opt, int& k_opt, 
		int* Np_opt, long double& PLRres, long double& RIres, long double Ptarget, long double Pe, long double Pf, int R, double rho){

//  	unsigned int Nrr, Nrr_max, k_max, tNrr, sum;
//  	unsigned int k;
//  	unsigned int n = 0;
//  	unsigned int t,c;
//  	unsigned int v;
//  	long double tPLRres, tRIres;
//
//	unsigned int Np[Nrr_max+1];           
//  	unsigned int Np_opt_temp[Nrr_max+1];
//	
//	initialize(Np_opt_temp, Nrr_max+1, 0, sizeof(int));
//	
//  	Nrr_max=floor((Dtarget - D_work - (double)(RTT/2.0))/(double)(RTT + D_work ));
//	
//	/*-------------------start with linear approximation---------------------*/
//
//	for(Nrr=0;Nrr<=Nrr_max;++Nrr){
//        	
//    		initialize(Np,Nrr+1,0,sizeof(int));
//    		k_max = floor(( Dtarget - D_work - (double) (RTT/2.0) - Nrr * ( RTT + D_work ))/Ts);
//                
//		for(k=1; k<=k_max && k_max!=1; k=k+k_max-1){
//
//			n = approximate_n_linearly(rho,Pe,k);
//  			tPLRres = 1;	  
//
//      			if( Nrr == 0 ){
//        	  		Np[0] = n - k;
//			}
//      			else if (Nrr >= 1) {	
//        	  		Np[0] = 0;
//
//        	  		for(c=1;c<Nrr;c++)
//					Np[c] = 1;
//				
//				if ( n-k-Nrr+1 >= 0 )
//        	  			Np[Nrr] = n - k - Nrr + 1;
//				else {
//					//approximated n is not enough. Re-estimate n = k + Nrr -1  
//					Np[Nrr] = 0;
//					n = k + Nrr - 1;
//				}
//      			}
//
//			tPLRres = PLRperformance(n,k,Pe,Np,Nrr+1,Pf,R,rho);
//			//std::cout << n << " " << k << std::endl; 
//	
//		      	while(tPLRres > Ptarget){
//		
//				if (Nrr == 0){
//				   	Np[0] += 1;
//					n += 1;
//				}
//			     	else{
//				  	Np[Nrr] += 1;
//					n += 1;
//				}
//			     	if ( n > m )
//				     break;
//		             	tPLRres = PLRperformance(n,k,Pe,Np,Nrr+1,Pf,R,rho);
//			}
//
//      			// Save current Optimum
//		        if (tPLRres <= Ptarget){
//				tRIres = RIperformance(R,n,k,Np,Nrr+1,Pe,rho);
//			    	if (tRIres < RIres){
//				    	k_opt = k;
//				    	//copy Np to Np_opt
//		                    	copyArray(Np,Np_opt_temp,Nrr+1);
//		                    	len_opt = Nrr+1;   
//		                    	RIres = tRIres;
//				    	PLRres = tPLRres;
//			    	}
//		
//		           	if (Nrr >= 1){
//		
//				    	//Greedy search errosion algorithm
//				    	for (tNrr = Nrr; tNrr> 0; tNrr--){ 
//		
//						while(Np[tNrr-1]+1 < Np[tNrr]){ 
//							Np[tNrr-1] = Np[tNrr-1] + 1;
//						    	Np[tNrr] = Np[tNrr] - 1;
//		                                        //tRIres = RIperformance(R,n,k,Np,Nrr+1,Pe,rho);
//					            	//Save current Optimum
//							
//						    	if (tRIres < RIres){
//								k_opt = k;
//		                                            	copyArray(Np,Np_opt_temp,Nrr + 1);
//		                                            	len_opt = Nrr + 1;   
//							    	RIres = tRIres;
//							    	PLRres = tPLRres;
//							}
//							
//					    	}
//				    	}	 
//			    	} 
//			}
//   		}
//	}
//
//	//Due to line approximation of n avoid passing Np[last_round] = 0
//
//	//if ( Np[len_opt-1]== 0 )               
//          //      len_opt=len_opt - 1;
//
//	copyArray(Np_opt_temp, Np_opt, len_opt);
}

// Compute Optimum values directly with out using linear approximation

void prrt_analysis::directOptimumComputation(float Dtarget, float DTx, float Dhost, float Dresp, float DPL, float Ts, float RTT, unsigned int Nrr_max, unsigned int m, unsigned int& len_opt, unsigned int& k_opt, unsigned int* Np_opt, double& PLRres, double& RIres, float Ptarget, float Pe, float Pf, unsigned int R, float rho){

  	unsigned int Nrr, k_max, tNrr, sum;
  	unsigned int k;
  	unsigned int n = 0;
  	unsigned int t,c;
  	unsigned int v;
  	long double tPLRres, tRIres;

	unsigned int Np[Nrr_max+1];           
  	unsigned int Np_opt_temp[Nrr_max+1];
	
	initialize(Np_opt_temp, Nrr_max+1, 0, sizeof(int));
	
	/*--------------------Start Without Linear Approximation----------------------*/

	//std::cerr << "Nrr_max " << Nrr_max << std::endl;

	for(Nrr=0; Nrr<=Nrr_max; ++Nrr){

		//std::cerr << "Nrr " << Nrr << std::endl;

		initialize(Np, Nrr+1, 0, sizeof(int));
		k_max=(unsigned int)fmaxf(1.0, fminf(20.0, floorf((Dtarget-DTx-(Dhost+RTT)/2.0-DPL-(float)(Nrr)*Dresp)/Ts)));
		//std::cerr << "k_max " << k_max << std::endl;

		for(k=1; k<=k_max && k_max!=1; k=k+k_max-1){
			//printf("k = %d, Nrr = %d, m = %d\n", k, Nrr, m);

			//std::cerr << "k " << k << std::endl;
			//increment last position of Np until target PLR is reached
			if(Nrr==0)
				Np[0]=0;
			else {	
				unsigned int c;
				Np[0]=0;
				for(c=1; c<Nrr; c++)
					Np[c]=1;
				Np[Nrr]=0;
			}
			tPLRres=1;
			while(tPLRres > Ptarget){ 
				if (Nrr==0)
					Np[0]+=1;
				else
					Np[Nrr]+=1;
				sum=0;		  
				for(t = 0; t<=Nrr; t++)
					sum+=Np[t];
				n=k+sum;
				if ( n > m)
					break;
				tPLRres=PLRperformance(n, k, Pe, Np, Nrr+1, Pf, R, rho);

				//std::cerr << "tPLRres " << tPLRres << std::endl;
				//printf("n=%d   k=%d   Nrr=%d   \n",n, k, Nrr);
				//printf("tPLRres= %f \n", (float)tPLRres);
			} 
			//printf("tPLRres= %Le \n",tPLRres);

			// Save current Optimum
			if (tPLRres <= Ptarget){

				tRIres = RIperformance(R, n, k, Np, Nrr+1, Pe, rho);
				//std::cerr << "tRIres " << tRIres << std::endl;
				//printf("n=%d   k=%d   Nrr=%d   m=%d   tRIres=%Lf \n",n, k, Nrr,m, tRIres);

				if (tRIres < RIres){
					//printf(" **tRIres=%Lf \n ",tRIres);
					k_opt=k;
					//copy Np to Np_opt
					copyArray(Np, Np_opt_temp, Nrr+1);
					//display(Np_opt,Nrr+1,sizeof(int));
					len_opt= Nrr+1;   //save current optimal length of Np_opt
					RIres=tRIres;
					PLRres=tPLRres;
				}

				if (Nrr >= 1){
					sum=0;
					for (t=0; t<=Nrr; t++)
						sum+=Np[t];
					n=k+sum;
					//Greedy search errosion algorithm
					for (tNrr=Nrr; tNrr>0; tNrr--){ 

						while(Np[tNrr-1]+1 < Np[tNrr]){

							//printf("hi \n");
							Np[tNrr-1]=Np[tNrr-1]+1;
							Np[tNrr]=Np[tNrr]-1;
							tRIres = RIperformance(R, n, k, Np, Nrr+1, Pe, rho);
							//std::cerr << "tRIres " << tRIres << std::endl;
							//printf(" **RIres=%Lf \n ", RIres);

							//Save current Optimum
							if (tRIres < RIres){

								k_opt=k;
								//printf("k_opt hii = %d \n ", k_opt);
								copyArray(Np, Np_opt_temp, Nrr+1);
								//display(Np_opt, Nrr+1, sizeof(int));
								len_opt= Nrr+1;   //save current optimal length of Np_opt
								RIres=tRIres;
								PLRres=tPLRres;
							}
						}
					}	 
				} 
			}
		}
	}

//	for(Nrr=0; Nrr<= Nrr_max; ++Nrr){
//
//    		initialize(Np,Nrr+1,0,sizeof(unsigned int));
//			k_max=fmin(20, floor((Dtarget-DTx-(Dhost+RTT)/2.0-DPL-(Nrr)*Dresp)/Ts));
//                
//		for(k=1; k<=k_max && k_max != 1; k=k+k_max-1){
//               
//        		//increment last position of Np until target PLR is reached
//      			if(Nrr==0)
//          			Np[0]=0;
//      			else {	
//          			unsigned int c;
//          			Np[0]=0;
//          			for(c=1;c<Nrr;c++)
//					Np[c]=1;
//          			Np[Nrr]=0;
//      			}
//		      	tPLRres=1;
//		      	while(tPLRres > Ptarget){ 
//				if (Nrr==0)
//				   	Np[0]+=1;
//			     	else
//				  	Np[Nrr]+=1;
//		             	sum=0;		  
//			     	for(t = 0;t<=Nrr;t++)
//			     	  	 sum+=Np[t];
//		             	n=k+sum;
//			     	if ( n > m)
//				     break;
//		             	tPLRres=PLRperformance(n,k,Pe,Np,Nrr+1,Pf,R,rho);
//			   } 
//
//      			// Save current Optimum
//
//		        if (tPLRres <= Ptarget){
//		
//				tRIres = RIperformance(R,n,k,Np,Nrr+1,Pe,rho);
//		
//			    	if (tRIres <= RIres){
//				    	k_opt = k;
//				    	//copy Np to Np_opt
//		                    	copyArray(Np,Np_opt_temp,Nrr+1);
//				    	//display(Np_opt,Nrr+1,sizeof(int));
//		                    	len_opt = Nrr+1;   //save current optimal length of Np_opt
//		                    	RIres = tRIres;
//				    	PLRres = tPLRres;
//			    	}
//		
//		           	if (Nrr >= 1){
//				    	sum=0;
//		                    	for (t=0;t<=Nrr;t++)
//						sum+=Np[t];
//				    	n=k+sum;
//					//Greedy search errosion algorithm
//				    	for (tNrr=Nrr;tNrr>0;tNrr--){ 
//		
//						while(Np[tNrr-1]+1 < Np[tNrr]){
//		
//						        //printf("hi \n");
//							Np[tNrr-1]=Np[tNrr-1]+1;
//						    	Np[tNrr]=Np[tNrr]-1;
//		                                        //tRIres = RIperformance(R,n,k,Np,Nrr+1,Pe,rho);
//							//printf(" **RIres=%Lf \n ",RIres);
//		
//					            	//Save current Optimum
//						    	if (tRIres < RIres){
//		
//								k_opt = k;
//								//printf("k_opt hii = %d \n ",k_opt);
//		                                            	copyArray(Np,Np_opt_temp,Nrr+1);
//		                                           	//display(Np_opt,Nrr+1,sizeof(int));
//		                                            	len_opt = Nrr+1;   //save current optimal length of Np_opt
//							    	RIres = tRIres;
//							    	PLRres = tPLRres;
//						    	}
//					    	}
//				    	}	 
//			    	} 
//			}
//   	    	}
//  	}

	copyArray(Np_opt_temp, Np_opt, len_opt);
}

double prrt_analysis::optimizeSingleParamSet(float Dtarget, float Ptarget, float CBw, float SBw, unsigned int R, float Pe, float RTT, float Pf, float rho, unsigned int LP, float Dhost, float DTx, float DPL, float DFB, float DSup){


 	float Ts, Ts_min;     //Packet interval Ts=[ms]
 	unsigned int m;
 	unsigned int q, w;  //loop variables

  double R_LFB = 0; // Loss feedback rate.
  double R_CFB = 0; // Channel feedback rate.
  double R_Source = 0;  // Source rate
  double RI = 0;        // Redundancy information.

	//std::cerr << "dtarget " << Dtarget << " CBw " << CBw << " SBw " << SBw << " DTx " << DTx << " Dhost " << Dhost << " DPL " << DPL << " DFB " << DFB << " DSup " << DSup << " Ts " << Ts << " RTT " << RTT << " Ptarget " << Ptarget << " Pe " << Pe << " Pf " << Pf << " R " << R << " rho " << rho << std::endl;

	//initialize the arrays Dist and FBdist
	initialize2D((long double *)Dist,-1,MAX_N,MAX_N);
 	initialize2D((long double *)FBdist,0,MAX_N,MAX_N); 	
 
 	Ts=(LP*8)/SBw;  
 	Ts_min=(LP*8)/CBw;  
 	m=(unsigned int) fminf(60.0, floorf(Dtarget/Ts_min));

	//std::cerr << "Ts_min " << Ts_min << " LP " << LP << " m " << m << std::endl;

//Fill out the array Dist till m x m

  if (m == 0) {
    // No solution for parameters calculation.
    return SBw;
  }
	
 	for(q=0; q<m; q++)
   		for(w=0; w<=q; w++)
      			Pdist(w, q, Pe, rho);
	
 	RI = optimizeHEC(Dtarget, Ptarget, CBw, SBw, R, Pe, RTT, Ts, Ts_min, Pf, rho, LP, Dhost, DTx, DPL, DFB, DSup); 

  // compute source rate R_Source and return to prrtsocket
  R_Source = ( SBw - R_LFB - R_CFB ) / (1 + RI + static_cast<double>(PRRT_DATA_HEADER_SIZE) / (LP) +
                   static_cast<double>(PRRT_RED_HEADER_SIZE) / (LP) * RI);

  return R_Source;
}

void prrt_analysis::copyArray(unsigned int *Np, unsigned int *Np_opt, unsigned int len){
     	unsigned int i=0;
     	for(i=0; i<len; i++)
        Np_opt[i]=Np[i];
}

void prrt_analysis::FBdistribution(unsigned int *Np, unsigned int len, float Pf, unsigned int R){
	
	long double prob[len];
	unsigned int sum[len];
	//long double FBdist[len][len];
	int i, j;
	unsigned int d=4; //repetition factor

        //Generate a diagonal binary matrix of size (len)x(len-1)
	if ( len-1 > 0 ){
		unsigned int binMatrix[len][len-1];

		for (i=0; i<len; i++){
			for (j=0; j<len-1; j++){
				if (j >= i)
					binMatrix[i][j]=0;
				else
					binMatrix[i][j]=1;
			}
		}
		if(len-1==1){
			prob[0]=powl(Pf, d*R);
			prob[1]=1-prob[0];
			}
		else{
			prob[0]=powl(Pf, (len-1+d-1)*R);
			j=1;
			for(i=len-1-1+d-1; i>=d; i--, j++)
				prob[j]=powl(Pf, i*R)*(1-Pf);
			prob[j]=1-powl(Pf, d*R);
			}

		//Sum = BinMat * Np(1, 2:m+1)';
                for (i=0; i<len; i++){
			sum[i]=0;
			for(j=0; j<len-1; j++)
				sum[i]+=binMatrix[i][j]*Np[j+1];
		}
				
	//Merge matricies sum[len] and prob[len] to FBdist=[sum prob]
		for(j=0; j<len; j++)
			FBdist[j][0]=sum[j];
		for(j=0; j<len; j++)
			FBdist[j][1]=prob[j];

			
	}

	else{
		FBdist[0][0] = 0;
		FBdist[0][1] = 1;
	}

}

void prrt_analysis::GEdistribution(float alpha, float beta, long double *PGE, unsigned int seq_len){

	//long double PGE[seq_len+1];
	long double PB, PG;  //Steady state probabilities
	long double P1, P2;
	unsigned int k, n;
	unsigned int counter, c_counter;
	long double t_counter;
	unsigned int bursts, gaps;
	unsigned int half_ceil;
	unsigned int NUM_SIZE, DENOM_SIZE;
	NUM_SIZE=2;
	DENOM_SIZE=4;
	unsigned int num[NUM_SIZE];
	unsigned int denom[DENOM_SIZE]; 

	initialize(PGE, seq_len+1, 0, sizeof(long double));
	PB=(1-beta)/(1-alpha+1-beta);
	PG=(1-alpha)/(1-alpha+1-beta);
	n=seq_len;
	t_counter=1;
	k=n;
	PGE[0]=PG*powl(beta, n-1);
	PGE[k]=PB*powl(alpha, n-1);
	
	half_ceil=ceil((double)n/2.0);
	for (bursts=1; bursts<=half_ceil; ++bursts){
		for (gaps=bursts-1; gaps<=bursts+1; ++gaps){
			if (gaps>0 && gaps<= half_ceil){
				for(k=bursts; k<=n-gaps; ++k){
						if (gaps==bursts){
							num[0]=k-1;num[1]=n-k-1;
							denom[0]=bursts-1;denom[1]=k-bursts;denom[2]=bursts-1;denom[3]=n-k-bursts;
							t_counter=fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE);
							P1= PG*powl(1-beta, bursts)*powl(1-alpha, bursts-1)*powl(beta, n-k-bursts)*powl(alpha, k-bursts);
							P2= PB*powl(1-alpha, gaps)*powl(1-beta, gaps-1)*powl(alpha, k-gaps)*powl(beta, n-k-gaps);
							PGE[k] += (t_counter * (P1 + P2));
							}
						else if (gaps==bursts+1){
							num[0]=k-1;num[1]=n-k-1;
							denom[0]=bursts-1;denom[1]=k-bursts;denom[2]=bursts;denom[3]=n-k-bursts-1;
							t_counter=fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE);
							P1= PG*powl(1-beta, bursts)*powl(1-alpha, bursts)*powl(beta, n-k-bursts-1)*powl(alpha, k-bursts);
							PGE[k] += (t_counter * P1);
							}
						else if (gaps==bursts-1){
							num[0]=k-1;num[1]=n-k-1;
							denom[0]=gaps;denom[1]=k-gaps-1;denom[2]=gaps-1;denom[3]=n-k-gaps;
							t_counter=fastfactorial(num, NUM_SIZE, denom, DENOM_SIZE);
							P1= PB*powl(1-alpha, gaps)*powl(1-beta, gaps)*powl(alpha, k-gaps-1)*powl(beta, n-k-gaps);
							PGE[k] += (t_counter * P1);
							}					
					}
				}
			}
		}
		//display(PGE, seq_len+1, sizeof(long double));
		//return &(PGE[0]);
}


void prrt_analysis::fill_num_denom(unsigned int *num, unsigned int NUM_SIZE, unsigned int *denom, unsigned int DENOM_SIZE, unsigned int n, unsigned int k, unsigned int i, unsigned int j){
	
	unsigned int q;
	for (q=0; q<DENOM_SIZE; q++){
		switch(q){

         		case 0:
            			num[q]=k;
            			denom[q]=n;
            			break;
         		case 1:
            			num[q]=n-k;
            			denom[q]=i;
            			break;
         		case 2:
            			num[q]=j;
            			denom[q]=k-i;
            			break;
         		case 3:
            			num[q]=n-j;
            			denom[q]=j-i;
            			break;
         		case 4:
            			denom[q]=n-k-j+i;
            			break;
       		}
   	}
}


int prrt_analysis::approximate_n_linearly(long double rho, long double Pe, int k){

  	int n=k;
	map<double, LineParam*>::iterator map_iter,rho_0_iter,rho_0_iter2;
	rho_0_iter = rho_0.begin();
        rho_0_iter2 = rho_0.lower_bound(0.211);

	//Pick the specific map using rho
  
	  if(rho == 0.0){
	
	          map_iter = rho_0.lower_bound(Pe);
	
		  //compute n = slope * k + inttercept
	
		  n = ( (*(rho_0[(*map_iter).first])).get_slope() ) * k +  (*(rho_0[(*map_iter).first])).get_intercept(); 
	  }
	  else if (rho > 0.0 && rho <= 0.05){
	  	
		  map_iter = rho_0_05.lower_bound(Pe);
	
		  //compute n = slope * k + inttercept
	
		  n = ( (*(rho_0_05[(*map_iter).first])).get_slope() ) * k +  (*(rho_0_05[(*map_iter).first])).get_intercept();
	  }
	  else {
	  	
		  map_iter = rho_0_1.lower_bound(Pe);
	
		  //compute n = slope * k + inttercept
	
		  n = ( (*(rho_0_1[(*map_iter).first])).get_slope() ) * k +  (*(rho_0_1[(*map_iter).first])).get_intercept();
	  }
	
	  // delete the seond object values of map 
	  return n;
  }


  double prrt_analysis::smoothPLR(double plr, double fall_acceleration, double max_deviation, double min_deviation)
  {
    static int fall_cnt = 0;
    static double last_plr = plr;
    static double last_smoothed_plr = plr;
    double diff = plr - last_plr;
    double smoothed_plr = last_smoothed_plr;
    // Use deviation to avoid smoothed PLR far from original one if PLR oscillating.
    double deviation = last_smoothed_plr - plr;
    
    if (min_deviation < deviation && deviation < max_deviation) {
      fall_cnt = 0;
    }
    else if (0 < diff && deviation < max_deviation) {
      if (diff < max_deviation) {
        smoothed_plr = last_smoothed_plr + diff;
      }
      else {
        // Avoid steep increasing when last smoothed plr is already greater than orginal one.
        smoothed_plr = plr + max_deviation;
      }
      fall_cnt = 0;
    }
    else if (diff < 0) {
      // Drop like free fall
      fall_cnt++;
      smoothed_plr = last_smoothed_plr - fall_acceleration * fall_cnt * fall_cnt;
      // Smoothed value should be not less than the orignal plr.
      if (smoothed_plr < plr) {
        smoothed_plr = plr;
        fall_cnt = 0;
      }
    }
    else {
      // two cases: (deviation >= max_deviation && diff >= 0) or (deviation <= min_deviation && diff = 0)
//       if (max_deviation <= deviation) {
//         smoothed_plr = last_smoothed_plr - fall_acceleration * fall_cnt * fall_cnt / 10;
//       }
//       fall_cnt = 0;
    }
    
    last_smoothed_plr = smoothed_plr;
    last_plr = plr;
    m_plr = smoothed_plr;

    // ensure non-zero PLR for online adaptation
    return fmax(0.02, smoothed_plr);
    //return smoothed_plr;
  }


long double prrt_analysis::computeRI(unsigned int num_recv, unsigned int k, const unsigned int *n_p, const unsigned int n_rr, float plr, float rho)
{
  unsigned int n = k;  // the length of the block code.
  for (unsigned int i= 0; i < n_rr; i++)
  {
    n += *(n_p + i);
  }
	initialize2D((long double*)Dist, -1, MAX_N, MAX_N);
  return RIperformance(num_recv, n, k, n_p, n_rr, plr, rho);
}
