/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef ERROR_H
#define ERROR_H

#include <string>
#include <exception>

#define CONN_SUCC	503	/*return value if a connect call was successful*/
#define BIND_SUCC	501	/*return value if a bind call was successful*/
#define NOEVENT		500	/*return value if no call was done but file got read*/

#define EGENERAL 	501	/*error code for errors produced when calling socket functions*/
#define EINVADDR 	502	/*error code for invalid ip addr*/
#define ENOSOCK 	503	/*error code when socket does not exist*/
#define EINVINP 	504	/*error code when input is not well formed*/
#define EINTERR		505	/*error code when system call was interrupted*/
#define EALLOC		506	/*error code when allocation failed*/

namespace media_transport_protocol {

class prrt_error {
protected:
    std::string reason;

public:
    prrt_error();
    prrt_error(const char *reason);

    ~prrt_error();

    const char *what();
};

// Classes for exception throwing.

class socket_error : public prrt_error {
public:
    socket_error();
    socket_error(const char* reason);
};

class module_error : public socket_error {
    int errorNumber;
public:
    module_error(const char* reason);
    module_error(const char* reason, int no);

    int GetErrorNumber() const {
        return this->errorNumber;
    };
};

class list_error : public socket_error {
public:
    list_error(const char* reason);
};

}

#endif /*ERROR_H*/
