/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "proc_file.h"
#include <cerrno>
#include <cstring>

using namespace media_transport_protocol;

proc_file::proc_file(const std::string &path)
throw(file_error)
{
	this->path = path;
	this->file = fopen(path.c_str(), "w+");
	if(!this->file) {
		std::string error = "Could not open proc file "+path;
		throw(file_error(error.c_str()));
	}
}

proc_file::proc_file(const std::string &path, bool write)
throw(file_error)
{
	std::string openmode = "r";
	if(write)
		openmode = "w";

	this->path = path;
	this->file = fopen(path.c_str(), openmode.c_str());
	if(!this->file) {
		std::string error = "Could not open proc file "+path;
		throw(file_error(error.c_str()));
	}
}

proc_file::~proc_file()
{
	fclose(this->file);
}

int proc_file::read(std::string &buffer)
throw()
{
	const int buffsize = 50;
	int read = 0;
	int ovread = 0;
	char readbuffer[buffsize];

	while(!feof(this->file)) {
		/*zero out buffer*/
		memset(readbuffer, '\0', buffsize);
		read = 0;

		/*read out buffsize data*/
		while(!feof(this->file) && read < buffsize) {
			read += fread(readbuffer+read, sizeof(*readbuffer), buffsize-read, this->file);
		}
		ovread += read;

		/*add the data to the string buffer*/
		buffer.append(readbuffer, read);
	}

	return ovread;
}

int proc_file::read(char *buffer, int len)
throw(read_error)
{
	int read = 0;

	while(!feof(this->file) && !ferror(this->file) && read < len) {
		read += fread(buffer+read, sizeof(*buffer), len-read-1, this->file);
		if(ferror(this->file)) {
			// Try again
			perror("Could not read proc file");
			return -1;
		}
	}

	if(!feof(this->file))
		throw(read_error("Buffer to small for read operation!"));
	else
		return read;
}

int proc_file::write(const std::string &buffer)
throw(write_error)
{
	int written = 0;

	written = fwrite(buffer.c_str(), 1, buffer.size(), this->file);
	if(written == EOF) {
		std::string error = "Could not write to proc file "+this->path;
		throw(write_error(error.c_str()));
	}

	return written;
}

int proc_file::write(const char *buffer, int len)
throw(write_error)
{
	int written = 0;

	written = fwrite(buffer, 1, len, this->file);
	if(written == EOF){
		std::string error = "Could not write to proc file "+this->path;
		throw(write_error(error.c_str()));
	}

	return written;
}

//
// These are the functions for the error types that are defined for this class
//
proc_file::proc_error::proc_error()
{}

proc_file::proc_error::proc_error(const char* reason)
{
	this->reason = reason;
}

proc_file::file_error::file_error(const char* reason)
{
	this->reason = reason;
}

proc_file::read_error::read_error(const char* reason)
{
	this->reason = reason;
}

proc_file::write_error::write_error(const char* reason)
{
	this->reason = reason;
}
