/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PROC_FILE_H
#define PROC_FILE_H

#include <string>
#include <cstdio>
#include "error.h"
#include <iostream>

namespace media_transport_protocol {

class proc_file {
	private:
		std::string path;
		FILE * file;

	public:
		// Classes for exception throwing.
		class proc_error : public prrt_error {
			public:
				proc_error();
				proc_error(const char* reason);
		};
		class file_error : public proc_error
			{ public: file_error(const char* reason);};
		class read_error : public proc_error
			{ public: read_error(const char* reason);};
		class write_error : public proc_error
			{ public: write_error(const char* reason);};

		proc_file(const std::string &path)
		throw(file_error);

		proc_file(const std::string &path, bool write)
		throw(file_error);

		~proc_file();

		int read(std::string &buffer)
		throw();

		int read(char *buffer, int len)
		throw(read_error);

		int write(const std::string &buffer)
		throw(write_error);

		int write(const char *buffer, int len)
		throw(write_error);

		int getfd()
		{return fileno(file);}
};

}

#endif /*PROC_FILE_H*/
