/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_PARAMS_H
#define PRRT_PARAMS_H

#include "prrt_params_base.h"
#include <string>
#include <vector>

namespace media_transport_protocol {


/*the value of this parameter is the targeted loss rate in promille*/
class param_PLR : public prrt_param {
private:
	int value;

public:
	param_PLR();
	param_PLR(const std::string &val);
	param_PLR(const int val);

	~param_PLR();


	std::string get_file_name() const;
	std::string get_strvalue() const;
};


class param_AHEC : public prrt_param {
private:
  unsigned k;
  std::vector<unsigned> n_p;
  int d_fec;  // Delay of FEC in ms.
  int d_c;  // Delay of block coding in ms.
  int d_req;  // Request delay for reactive repair cycles.
  int d_fb;   // Delay of feedback in ms, after which the receiver sends the feedback.
  int d_target; // Target delay in ms.
public:
  param_AHEC();
  param_AHEC(int d_target, int d_fec, int d_c, int d_req, int d_fb, unsigned k, const std::vector<unsigned> &n_p);

  ~param_AHEC();

  void set_k(unsigned k);
  unsigned get_k() const;
  void set_n_p(const std::vector<unsigned> &n_p);
  std::vector<unsigned> get_n_p() const;
  void set_n_p(const std::string &n_p);
  void set_d_fec(int d_fec);
  int get_d_fec() const;
  void set_d_c(int d_c);
  int get_d_c() const;
  void set_d_req(int d_req);
  int get_d_req() const;
  void set_d_fb(int d_fb);
  int get_d_fb() const;
  void set_d_target(int d_target);
  int get_d_target() const;

  std::string get_file_name() const;
  std::string get_strvalue() const;

  void add_n_p(unsigned n_p);
};


class param_BW_est : public prrt_param {
private:
	int value;

public:
	param_BW_est();
	param_BW_est(const std::string &val);
	param_BW_est(const int val);

	~param_BW_est();

	std::string get_file_name() const;
	std::string get_strvalue() const;
};

class param_BUF_fb : public prrt_param {
private:
	int value;

public:
	param_BUF_fb();
	param_BUF_fb(const std::string &val);
	param_BUF_fb(const int val);

	~param_BUF_fb();

	std::string get_file_name() const;
	std::string get_strvalue() const;
};

}

#endif /*PRRT_PARAMS_H*/
