/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef SOCKET_LIST_H
#define SOCKET_LIST_H

#include <list>

#include "error.h"

namespace media_transport_protocol {

class prrt_socket;

class socket_list {
	private:
		std::list<prrt_socket *> sock_list;

	public:
		class list_error : public prrt_error
			{ public: list_error(const char *reason) { this->reason = reason; }; };

		socket_list();
		~socket_list();

		/**
		* Add a socket to the socketlist.
		* \param socket the socket object that should be added.
		*/
		void add(prrt_socket *socket);

		/**
		* Remove a random socket from the list.
		* \return the socket that was deleted from the list.
		*/
		prrt_socket *del_rnd()
		throw(list_error);

		/**
		* Remove a socket from the list.
		* \param socket the socket object that should be removed.
		*/
		void del(prrt_socket *socket);
};

}

#endif /*SOCKET_LIST_H*/
