/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "tfrc.h"

#include <sstream>
#include <iostream>

using namespace media_transport_protocol;
using namespace std;

prrt_tfrc::prrt_tfrc() {
	//Loss interval history
	for (char i = 0; i<9; i++)
	loss_interval[i]=0;
	
	//loss interval history pointers
	li_old=-1; 
	li_new=-1;
	
	//implement the last 8 loss interval history using linked list
	num_intervals=1;
	
	//weights for the last 8 loss intervals
	for (char i = 0; i<4; i++)
		weights[i]=1;
	weights[4] = 0.8;
	weights[5] = 0.6;
	weights[6] = 0.4;
	weights[7] = 0.2;
	
	//history discounting weight
	for (char i = 0; i<7; i++)
	hist_disc[i]=-1;
	
	//history discounting pointers
	hd_old=-1; 
	hd_new=-1;
	
	//number of history discount factors
	num_hist_disc=0;

	//average loss interval
	avg_loss_int=0;

        for(char i=0; i<HISTORY_LEN; i++)
		pkt_recd_hist[i]=-1;

	std_dev=0;
	sd_pointer=-1;

	for(char i=0; i<HISTORY_LEN; i++)
		T_hist[i]=-1;

	sd_pointer=-1;

	state = SS_STATE;

	tot_pkts=0;
	
}

prrt_tfrc::~prrt_tfrc() {
}

//to calculate the average loss interval
double prrt_tfrc::calc_avg_loss_interval(const char * loss_sequence)
{
	//here the loss interval array would be populated before calculating the avg loss interval
	//every time loss event occurs, update the loss interval array, and the hist_disc array
	
	char ch;
	int i;
	
	std::cout<< "Loss sequence \t";
	
	bool First;
	First = true;

	for(i=0;loss_sequence[i]!='\0';i++)
	{	
		std::cerr<<loss_sequence[i]<<" ";

		//if pkt lost and it is the first one, then do the shifting of history
		if(loss_sequence[i]!=PKT_ARRIVED && First==true)
		{
			//shift the elements of loss_interval
				
			li_new = ((li_new + 1) % 9);
				
			if(li_new==li_old)
			li_old = ((li_old +1) % 9);
				
			//reset the oldest hist_disc only when an interval goes out of the history
			if(num_intervals==9)
			{
				hist_disc[hd_old]=-1;
				hd_old=((hd_old+1)%7);
			}
			
			else if(num_intervals <9)
				num_intervals++;
				
			//put s0 as 0
			loss_interval[li_new]=0;

			First=false;
		}

		else
		{
			if(li_new==-1)
			{
				li_new=0;
				li_old=0;
			}
			
			loss_interval[li_new]++;
			
		}

	}
	
	weights[7]=0.2;

	std::cout<< std::endl;

	std::cerr << "Loss interval \t";

	for(i=0;i<num_intervals;i++)
	 std::cerr << loss_interval[i] << " ";
	
	std::cerr << std::endl;

	if (num_intervals == 1)
  	{
		avg_loss_int=loss_interval[li_new];
		return avg_loss_int;
	}
	
	//now calculate the average loss interval based on the final set of loss intervals in the history	
	//take all cases into account, history discounting etc, and find, final s_avg value
	
	//if s0 > = 2 * avg_loss_int, then set the new history discounting weights //to add to hist_disc do new ++
	if(loss_interval[li_new] >= (2 * avg_loss_int))
	{
		
		double d_wt,temp;
		temp=(2.0 * avg_loss_int);
		temp = temp/(double)(loss_interval[li_new]);
		d_wt=(0.5 > temp ? 0.5 : temp);

		if(hd_new==-1)
		{
			//-1 for interval s1 onwrds and another -1 for array indexing
			hd_new=num_intervals-2;
			hd_old=0;
			
			for(i=0;i<=hd_new;i++)
			{
				hist_disc[i]=d_wt;
			}
		}
		
		else 
		{
			
			int count=0;
			i=hd_old;
			while(count<num_intervals-1)
			{
				if(hist_disc[i]==-1)
					hist_disc[i]=d_wt;
				
				hd_new=i;
				i=(i+1)%7;
				count++;
			}
			
			if(num_intervals==9)
				hd_new=((hd_old+6)%7);
			
		}
	}
				
	
	//to calculate the average using the loss interval history and the history discounting weights
	double s_avg_1, s_avg_0, num=0, den=0, dummy_wt=1;
	int li_cnt, hd_cnt;
	
	//calculating without s0, s_avg_1
	
	num=0;
	den=0;
	li_cnt=li_old;
	hd_cnt=hd_old;
	
	//if loss_interval history is not full as yet
	if(num_intervals<9)
	{
		for(i=num_intervals-2;i>=0;i--)
		{
			dummy_wt=hist_disc[hd_cnt];
			if(dummy_wt==-1 || hd_cnt == -1)	
				dummy_wt=1;
			
			den += (dummy_wt * weights[i]);
			
			num += (dummy_wt * weights[i] * loss_interval[li_cnt]);

			li_cnt=(li_cnt+1)%9;
			hd_cnt=(hd_cnt+1)%7;
		}
		s_avg_1= num/den;
	}
				   
	
	//if num_intervals == 9, full loss history
	else
	{		
		for(i=7;i>=0;i--)
		{
			dummy_wt=hist_disc[hd_cnt];
			//if hist wt not set, or if the latest interval being processed, wt =1
			if(dummy_wt==-1 || i==0 || hd_cnt == -1)	
				dummy_wt=1;
			
			den = den + (dummy_wt * weights[i]);
			
			num += (dummy_wt * weights[i] * loss_interval[li_cnt]);
			
			li_cnt=(li_cnt+1)%9;
			hd_cnt=(hd_cnt+1)%7;
			
		}
		s_avg_1= num/den;
	}
	
	//calculating with s0, s_avg_0
	
	//if loss_interval history is not full as yet
	if(num_intervals<9)
	{
		num=0;
		den=0;
		li_cnt=li_old;
		hd_cnt=hd_old;
		
		for(i=num_intervals-1;i>=0;i--)
		{
			dummy_wt=hist_disc[hd_cnt];
			if(dummy_wt==-1 || hd_cnt == -1|| hd_cnt == -1)	
				dummy_wt=1;
			
			den += (dummy_wt * weights[i]);
			
			num += (dummy_wt * weights[i] * loss_interval[li_cnt]);
			
			li_cnt=(li_cnt+1)%9;
			hd_cnt=(hd_cnt+1)%7;
		}
	
		s_avg_0= num/den;
	}
	
	//if num_intervals == 9, full loss history
	else
	{
		num=0;
		den=0;
		li_cnt=(li_old+1)%9;
		hd_cnt=hd_old;
		
		for(i=7;i>=0;i--)
		{
			
			dummy_wt=hist_disc[hd_cnt];
			//if hist wt not set, or if the latest interval being processed, wt =1
			if(dummy_wt==-1 || i==0 || hd_cnt == -1)	
				dummy_wt=1;
			
			den += (dummy_wt * weights[i]);
			
			num += (dummy_wt * weights[i] * loss_interval[li_cnt]);
			
			li_cnt=(li_cnt+1)%9;
			hd_cnt=(hd_cnt+1)%7;
			
		}
		
		s_avg_0= num/den;
	}

	
	std::cerr<< "\t li_old= " <<li_old<< "\t li_new= " << li_new;
	std::cerr << std::endl;	

	std::cerr<<"Hist_disc \t";

	for(i=0;i<7;i++)
	 std::cerr << hist_disc[i] << " ";
	 
	std::cerr<< "\t hd_old= " <<hd_old<< "\t hd_new= " << hd_new;
	std::cerr << std::endl;	
		
	std::cerr << "s_avg_1 " << s_avg_1 << std::endl;	
	std::cerr << "s_avg_0 " << s_avg_0 << std::endl;
	std::cerr << "num_intervals " << num_intervals << std::endl;
		

	avg_loss_int=(s_avg_0>s_avg_1?s_avg_0:s_avg_1);
	return avg_loss_int;
	
}


//to calculate the TCP throughput T as a function of RTT and loss event rate
unsigned int prrt_tfrc::calc_T(double RTT, const char * loss_sequence, double min_pkt_int, double feedback_interval)
{	
	int i;

	if(min_pkt_int==0)
		min_pkt_int=1;
	//bottleneck bandwidth calculation using min_pkt_int
	//this is observed rate at the receiver
	double bn_bw= (PKT_SIZE * 8) / min_pkt_int;

	//to check if there are pkt losses
	for(i=0;loss_sequence[i]!='\0';i++)
	{
		//if a pkt is lost, then reinitialize tot_pkts to 0, and set state to NM_STATE
		if(loss_sequence[i]!=PKT_ARRIVED)
		{
			tot_pkts=0;
			//if it is a transition from SS to NM, then reinitialize
			//loss interval history
			if(state == SS_STATE)
			{
				// re-initialize the loss interval history
				
				//Loss interval history
				for (char i = 0; i<9; i++)
				loss_interval[i]=0;
	
				//loss interval history pointers
				li_old=-1; 
				li_new=-1;
	
				//implement the last 8 loss interval history using linked list
				num_intervals=1;
				
			}
			state=NM_STATE;
			
		}
		else
		{
			tot_pkts++;
		}
	}
	
	if(tot_pkts > PKT_THRESHOLD)
		state = SS_STATE;
	
	//if in slow start state, then send back SS_FACTOR * bn_bw to sender based
	//on the min pkt interval at receiver
	//get back to Slow start if for a long time no packets are lost
	//i.e. PKT_THRESHOLD pkts are received
	if(state == SS_STATE)
	{
		tot_pkts=0;
		std::cerr<<"SS bn_bw = "<< bn_bw<< std::endl;
		
		bn_bw = bn_bw * SS_FACTOR;

		if(bn_bw > UPPER_T)
		{	
			std::cerr << "bn_bw Bound exceeded " << bn_bw << std::endl;
			bn_bw=UPPER_T;
			std::cerr<<"After re-assignment bn_bw = " << bn_bw << std::endl;
		}
		std::cerr<<"SS bn_bw * SS_FACTOR= "<< bn_bw<< std::endl;
		return bn_bw;
	}
	
	std_dev = calc_std_dev(loss_sequence, feedback_interval);

	//to convert the RTT in milli seconds to seconds

	//TCP Retransmit timeout
	double t_RTO = 4 * RTT;
	
	//TCP Throughput (Available BW)
	double T; 
		
	//Average Packet size 
	double pkt_size=1356;
		
	//Avergae loss interval
	double avg_loss_int;
	
	//Loss event rate
	double loss_event_rate;

	unsigned int t;
	
	//Call function to calculate the avg loss interval 
	avg_loss_int=calc_avg_loss_interval(loss_sequence);

	//Loss event rate is the inverse of the aveage loss interval
	loss_event_rate = (double) (1.0/avg_loss_int);

	double temp1, temp2, temp3;
	temp1=(RTT * sqrt(2* loss_event_rate / 3));
	temp2=t_RTO * 3 * sqrt(3*loss_event_rate/8) * loss_event_rate;
	temp3=( 1 + (32 * loss_event_rate * loss_event_rate));
	 
	//Kbytes/second
	T= (pkt_size)/( temp1 + ( temp2 * temp3 ));
	
	//converting to bits/second	
	T= T * 8;
	//converting to Kbit/second
	T = T / 1000;

	if(T > UPPER_T)
	{	
		std::cerr << "T Bound exceeded " << T << std::endl;
		T=UPPER_T;
		std::cerr<<"After re-assignment T = " << T << std::endl;
	}
	//insert the calculated T into the history
	th_pointer= (th_pointer + 1)% HISTORY_LEN;
	T_hist[th_pointer] = T;

	//calculate the average T

	int count=0,th_count;
	double total=0;

	if(th_pointer==HISTORY_LEN -1)
		th_count=0;
	else	
		th_count=th_pointer+1;


	for(i=0; i<HISTORY_LEN ; i++)
	{
		if(T_hist[th_count]!= -1)
		{
			total = total + T_hist[th_count];  
			count++;
		} 
		th_count = (th_count + 1) % HISTORY_LEN;
	}

	//average value of T
	if(count!=0)
  	T = total / count;
	
	//accounting for the standard deviation and taking the lower value
	if(T>=std_dev)	
		T = T - std_dev;	
	std::cerr<<"TFRC est = "<<T<<std::endl;

	t= (unsigned int) T;
	return t;
}

double prrt_tfrc::calc_std_dev(const char * loss_sequence, double feedback_interval)
{
	int num_pkts=0;
	for(int i=0;loss_sequence[i]!='\0';i++)
	{
		if(loss_sequence[i]==PKT_ARRIVED)
			num_pkts++;
	}
	
	sd_pointer= (sd_pointer + 1)% HISTORY_LEN;
	pkt_recd_hist[sd_pointer] = num_pkts;sd_pointer= (sd_pointer + 1)% HISTORY_LEN;
	pkt_recd_hist[sd_pointer] = num_pkts;

	//calculation of the standard deviation depending on current history
	
	double Bf_d = 0, Bf=0, temp=0, dev =0 ;
	int pkt_size = PKT_SIZE, count=0;
	int sd_count;

	if(sd_pointer==HISTORY_LEN -1)
		sd_count=0;
	else	
		sd_count=sd_pointer+1; 

	//to calculate the mean of the time series
	for(int i=0; i<HISTORY_LEN ; i++)
	{
		if(pkt_recd_hist[sd_count]!= -1)
		{
			Bf = Bf + (pkt_recd_hist[sd_count] * pkt_size);  
			count++;
		} 
		sd_count = (sd_count + 1) % HISTORY_LEN;
	}
	
	//mean
	Bf = Bf/(HISTORY_LEN * feedback_interval);

	if(sd_pointer==HISTORY_LEN -1)
		sd_count=0;
	else	
		sd_count=sd_pointer+1; 

	count =0;
	for(int i=0; i<HISTORY_LEN ; i++)
	{
		if(pkt_recd_hist[sd_count] != -1)
		{
			count++;
			temp = temp + (pkt_recd_hist[sd_count] * pkt_size);
			//throughput till each updation interval
			Bf_d = temp / ((i+1) * feedback_interval );
	
			//sum of squares of the differences to mean
			dev = dev + ((Bf_d - Bf) * (Bf_d - Bf));
		}
		sd_count = (sd_count + 1) % HISTORY_LEN;	
	}
	
	//standard deviation
	std_dev =  sqrt(dev/ HISTORY_LEN);

	return std_dev;
}
