/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef TFRC_H
#define TFRC_H

#define PKT_ARRIVED '0'
#define HISTORY_LEN 5
#define PKT_SIZE 1356
//maybe should be put equal to 2, ensuring that we are friendly towards TCP
#define SS_FACTOR 2
#define SS_STATE 0
#define NM_STATE 1
//set this to get back to the Slow Start State
#define PKT_THRESHOLD 20000
//to put upper bound on the rate 30 Mbps
#define UPPER_T 30000


#include <string>
#include <math.h>

namespace media_transport_protocol {

class prrt_tfrc {
public:
    prrt_tfrc();

    ~prrt_tfrc();

    unsigned int calc_T(double RTT, const char * loss_sequence, double min_pkt_int, double feedback_interval=1);
private:

    double calc_avg_loss_interval(const char * loss_sequence);

    double calc_std_dev(const char * loss_sequence, double feedbac_interval);
    //Loss interval history
    int loss_interval[9];

    //loss interval history pointers
    int li_old, li_new;

    //implement the last 8 loss interval history using linked list
    int num_intervals;

    //weights for the last 8 loss intervals
    float weights[8];

    //history discounting weight
    float hist_disc[9];

    //history discounting pointers
    int hd_old, hd_new;

    //number of history discount factors
    int num_hist_disc;

    //average loss interval
    double avg_loss_int;

    //history of packets received for std dev calc
    int pkt_recd_hist[HISTORY_LEN];

    //std dev history pointer to the newest element
    int sd_pointer;//sd_new, sd_old;

    //standard deviation of the bandwidth estimate 
    double std_dev;

    //state : Slow Start or Normal
    int state;
	
    //total number of packets received till now
    long tot_pkts;

    //history of T
    double T_hist[HISTORY_LEN];

    //pointers to newest element in T_Hist
    int th_pointer;
};

};
#endif /*TFRC_H*/
