/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_RECEIVER_H
#define PRRT_RECEIVER_H

#include <iostream>
#include <fstream>
#include <string>
#include <prrt/prrt_sockets.h>

/**
 * This class creates an prrt receiver.
 */
class prrt_receiver {
	public:
		prrt_receiver();
		/// This constructor is for unicast purposes only.
		/// It sets up address, feedback address, port and file to stream and connects
		/// the underlying prrt socket.
		/// \param addr the address of this host.
		/// \param port port receive from.
		/// \param path path of the file to save the data.
		prrt_receiver(std::string addr, unsigned int port, std::string path);
		prrt_receiver(std::string addr, std::string fbaddr, unsigned int port, std::string path);

		~prrt_receiver();

		/// Set the host IP address for the connection.
		/// For multicasting set this address to the multicast address.
		void set_address(std::string addr);

		/// Set the IP address for feedback handling.
		/// For multicasting set this to "0".
		void set_fbaddress(std::string addr);

		/// Set the port for this connection.
		/// In fact port+1 will be used too.
		void set_port(unsigned int port);

		/// Create socket and bind it.
		void setup_socket();

		/// Set file that should be streamed.
		void set_file(std::string path);

		/// Start receiving data.
		/// This function will block the program until it is terminated.
		/// The received data is streamed into the given file.
		void recv();

	private:
		media_transport_protocol::prrt_socket msock;
		std::string file_path;

		// information concerning the prrt connection
		std::string address;
		std::string fbaddress;
		unsigned int port;
};

#endif /*PRRT_RECEIVER_H*/
