/*
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef DEFINES_H
#define DEFINES_H


#include <linux/version.h>

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
#include <linux/param.h>
#endif

// the new kernel version removed these defines
#ifndef init_MUTEX
#define init_MUTEX(sem) sema_init(sem, 1)
#endif /*init_MUTEX*/

#ifndef init_MUTEX_LOCKED
#define init_MUTEX_LOCKED(sem) sema_init(sem, 0)
#endif /*init_MUTEX_LOCKED*/


#include "doc_helps.h"
//#include "prrt_socket.h"
#include <linux/types.h>

typedef int Socketno_t;
typedef int Relayno_t;

#define UNIT_TEST 0

#define GF_BITS 8	/*the number of bits for one symbol in the galois field*/

#define TYPEII_HEC_MODE 1 /*switch on/off Type II HEC mode (PRRT operates according to GT's statistical framework when switched off).*/

#define PROC_DIR prrt	/*directory for all the proc files of this program*/

#define RELAYNAMELENGTH 20  /*the length of the name of a relay*/

#define MSG_DEBUG_ON 0	/*should debug messages be turned on? (detailed settings below)*/

#define SIMULATION_ON_SEND 0	/*decide whether LOSS_RATE should be simulated*/
#define DROP_PACKS_ON_SEND 0	/*should data packets be dropped*/
#define DROP_FBPACKS_ON_SEND 0	/*should feedback packets be dropped*/
#define DROP_REPACKS_ON_SEND 0	/*should replacement packets be dropped*/

#define SIMULATION_ON_RECV 0 /*should arriving packets be dropped with LOSS_RATE*/

#define MSG_TRACE 0	/*Should trace infos by printed*/

#define DEBUG 1		/*currently debugging?(1=YES, 0=false)*/

#define NAME_SIZE 35	/*size of the directory names under proc*/

#define MAXFBS 16	/*maximum number of feedbacks per packet,atm 16 (do not change!)*/
#define MAXFBPACKS 10	/*maximum number of feedback packet to build*/

#define USE_FEEDBACK_STALLING 0/*should feedback be stalled until first redundancy arrives*/

/*
 * ----------------------------------------------------------------------------
 * Timer values for work queues and sockets
 * ----------------------------------------------------------------------------
 */
#define POLLDELAYMS 5			/*value in ms to wait for next poll*/
#define POLLDELAY POLLDELAYMS*HZ/1000	/*value in jiffies*/

#define POLL_INIWAITTIMEMS 1000		/*first waittime when waiting for
					 *arriving packet in ms*/
#define POLL_MAXWAITTIMEMS 100		/*max waittime when waiting for
					 *arriving packet in ms*/
#define POLL_INIWAITTIME POLL_INIWAITTIMEMS*HZ/1000
#define POLL_MAXWAITTIME POLL_MAXWAITTIMEMS*HZ/1000

#define SENDDELAYMS 50			/*value in ms to wait for next send*/
#define SENDDELAY SENDDELAYMS*HZ/1000	/*value in jiffies*/

#define SENDDELDELAYMS 50		/*value in ms to wait for next
					 *deletion for block queue*/
//#define SENDDELDELAY (atomic_read(&msock->inc_packet_list.D_target)/10)	/*value in jiffies*/
#define SENDDELDELAY SENDDELDELAYMS*HZ/1000

#define FEEDBACKDELAYMS 15		/*value in ms to wait for next feedback*/
#define FEEDBACKDELAY FEEDBACKDELAYMS*HZ/1000	/*value in jiffies*/

#define REPLACEMENTDELAYMS 5		/*value in ms to wait for a new round of replacements*/
#define REPLACEMENTDELAY REPLACEMENTDELAYMS*HZ/1000	/*value in jiffies*/

#define READPKTDELAYMS 10		/*value in ms to wait for a new round of replacements*/
#define READPKTDELAY READPKTDELAYMS*HZ/1000	/*value in jiffies*/

#define POLLTIMEOUT 0		/*time in seconds after a timeout happens
				 *on a receive*/
#define POLLTIMEOUTUS 100000	/*time in seconds after a timeout happens
				 *on a receive*/

#define POLLTIMEOUTFB 3		/*time in seconds after a timeout happens
				 *on a feedback receive*/

/*maximum delay between last block packet and newest packet in outgoing queue*/
#define SENDMAXDELAY 1000/*((atomic_read(&msock->inc_packet_list.D_target))
+2*(atomic_read(&msock->inc_packet_list.RTT)))*/

#define WORKDELAYMS 30	/*this is the work delay between receiving packets and sending packets*/

/*packet may be send after MAYREAD_DELAY_MULTI*D_TARGET+MAYREAD_DELAY_ADD*/
#define MAYREAD_DELAY_MULTI	2
#define MAYREAD_DELAY_ADD	1

/*packet may be deleted after MAYDELPACK_DELAY_MULTI*D_TARGET+MAYDELPACK_DELAY_ADD*/
#define MAYDELPACK_DELAY_MULTI	4
#define MAYDELPACK_DELAY_ADD	300

#define MAYDELBLOCK_DELAY_ADD	200

#define MAYSENDFB_BLOCK_MULTI	2

/*
 * Testing options
 */
#define USE_FIXED_T_REPLACE 0

/*
 * ----------------------------------------------------------------------------
 * Start values for protocol parameters
 * ----------------------------------------------------------------------------
 */
#define D_TARGET_STARTMS 100
#define D_FEC_STARTMS 40
#define D_C_STARTMS 20
#define D_REQ_STARTMS 40
#define D_TARGET_START D_TARGET_STARTMS*HZ/1000

#define PLR_TARGET_START 5	/*in promille*/
#define BW_EST_START 1000	/*in Kbit/s*/
#define SENDING_RATE 10 /*in Mbit/s*/


#define RTT_STARTMS 200
#define RTT_START RTT_STARTMS*HZ/1000

#define K_START 2	/*must be >0*/
#define N_START 4	/*should be >k to be save, must be >=k*/
#define N_P_START 1	/*number of replacements in first round*/
#define N_CC_START 1

#define CODE_START 2	/*this is the number of packets we code when the block is created*/

#define MAX_BURST 200      /*do not try to correct error bursts longer than MAX_BURST*/

/*
 * ----------------------------------------------------------------------------
 * Start values for simulation purposes
 * ----------------------------------------------------------------------------
 */
#define LOSS_RATE 5


/*
 * ----------------------------------------------------------------------------
 * Macros
 * ----------------------------------------------------------------------------
 */

#define MAX(a,b) ((a) > (b) ? (a) : (b))	/*this macro gives us the
						 *maximum value of a and b*/

#define MIN(a,b) ((a) < (b) ? (a) : (b))	/*this macro gives us the
						 *minimum value of a and b*/

#define jif_to_ms(jif) \
((u32)((1000/HZ)*jif))

#define ms_to_jif(ms)\
((u32)((ms)*HZ/1000))

#define INIT_MSGHDR_MEMBERS(MSG) \
MSG.msg_name = 0; \
MSG.msg_namelen = 0; \
MSG.msg_iov = 0; \
MSG.msg_iovlen = 0; \
MSG.msg_control = 0; \
MSG.msg_controllen = 0; \
MSG.msg_flags = 0;

#define INIT_MSGHDR(MSG) \
struct msghdr MSG; \
INIT_MSGHDR_MEMBERS(MSG)

#if MSG_TRACE
	#define PRINT_TRACE(entity, proc, event) \
	printk(KERN_ALERT "prrt: Trace %s %s: %s\n", entity, proc, event)
#else
	#define PRINT_TRACE(entity, proc, event)
#endif

/*these are tags for different debugging messages*/
#define MSG_COMMON	-1	/*general messages*/
#define MSG_COMMONRECV 	0	/*general messages of the receiving part*/
#define MSG_COMMONSEND 	1	/*general messages of the sending part*/
#define MSG_INIT 	2
#define MSG_TEST 	3
#define MSG_FATAL 	4	/*messages for fatal errors*/
#define MSG_BCDEB 	5	/*debugging messages for block coding*/
#define MSG_BLOCKDEB 	6	/*debugging messages for blocks*/
#define MSG_SENDFBDEB 	7	/*debugging messages for sending feedback*/
#define MSG_RECVFBDEB	20	/*debugging messages for receiving feedback*/
#define MSG_FBADEB	8	/*debugging messages for feedback analysis*/
#define MSG_CLOCKDEB	9	/*debugging messages for estimated sender clock*/
#define MSG_DECODDEB	10	/*debugging messages for decoding*/
#define MSG_PACK	11	/*debugging messages for packets*/
#define MSG_PARAM	12	/*debugging messages for parameter estimation*/
#define MSG_SENDBLOCK	13	/*debugging messages for sending blocks*/
#define MSG_PRINTDATA	14	/*printing the contents of a packet*/
#define MSG_ALLOC	15	/*show allocation and deletion of memory*/
#define MSG_FINAL	16	/*print final stats for packet transmission*/
#define MSG_TRANSPORT	17	/*debugging messages about packet transport bla*/
#define MSG_STATS	18	/*debugging messages for statistics*/
#define MSG_RECV	19	/*debugging messages for receiving packets of the stream*/
#define MSG_KUIPARAM	21	/*debugging messages about setting parameters through proc fs*/
#define MSG_PRRTRECV	22	/*debugging messages for the prrt_recv function*/
#define MSG_INITSEND	23	/*debugging messages for the initalisation of the sending part*/
#define MSG_SENDDEB	24
#define MSG_FBANADEB	25	/*debugging messages for the feedback analyzer*/
#define MSG_BLOCKLIST	26
#define MSG_CODING	27
#define MSG_DATALIST	28
#define MSG_INCLIST	29
#define MSG_OUTLIST	30
#define MSG_KUI		31
#define MSG_KERN_WRAP	32
#define MSG_KUICONTROL	33
#define MSG_FBSTALLING	34
#define MSG_SENDREPL	35
#define MSG_PACKMISSES  36
#define MSG_PACKOMMIT	37
#define MSG_PACKRETURN	38
#define MSG_LISTSTATS	39
#define MSG_STABILITY   40

#define PRINT_PACKETHEX 50
#define PRINT_PARAMS	51
#define PRINT_RECVWORK  52

// turn on/off certain debugging messages by inserting deleting the cases for them
// case PRINT_PARAMS: case MSG_SENDFBDEB:
//	case MSG_RECVFBDEB: case MSG_FBANADEB: case MSG_PACKMISSES:
	//case MSG_FINAL: case MSG_KUICONTROL: case MSG_KUIPARAM: case MSG_TRANSPORT: case MSG_FBSTALLING: case MSG_PACKMISSES: case MSG_RECVFBDEB: case MSG_FBANADEB: case MSG_SENDFBDEB: 
//case MSG_KUICONTROL: case MSG_KUIPARAM: case MSG_FINAL: case MSG_FBANADEB: case MSG_SENDREPL: 

#undef PDEBUG
#if MSG_DEBUG_ON
	#define PDEBUG(tag, fmt, args...) \
	switch(tag) { \
case MSG_KUIPARAM: \
		printk(KERN_EMERG "prrt: " fmt, ## args); \
		break; \
	}
#else
	#define PDEBUG(fmt, args...)
#endif

#undef PDEBUGG
#define PDEBUGG(tag, fmt, args...)	/*placeholder to comment prints*/

#define PERROR(fmt, args...) \
printk(KERN_EMERG "prrt ERROR: " fmt, ## args);

#define PLOGPACK(fmt, args...) \
printk(KERN_EMERG "prrt Packet Log: " fmt, ## args);

#define bcopy(s, d, siz)	memcpy((d), (s), (siz))
#define bzero(d, siz)		memset((d), '\0', (siz))
#define bcmp(b1, b2, siz)	memcmp((b1), (b2), (siz))

#endif /*DEFINE_H*/
