/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef DOC_HELPS_H
#define DOC_HELPS_H

/*
 * These defines are meant to document certain issues about structs or
 * variables (or pseudo classes).
 * Some of the terms only apply to classes. As this (C) is not an object
 * oriented programming language, the term pseudo object/class is introduced.
 * Therefore we define a pseudo class as a struct and a set of functions that
 * operate on this struct.
 * Generally you are only assumed to change __PUBLIC fields of the struct.
 * __PRIVATE fields should only be accessed by the given function set that is
 * declarated together with the struct.
 * NOTE: To conform to these assumptions is completely up to the programmer,
 *       so please take it seriously.
 */

/*
 * This defines a pseudo class.
 */
#define __PCLASS

/*
 * This class extends the given class.
 */
#define __EXTENDS(class)

/*
 * These define sections in the struct with public and private fields.
 */
#define __PUBLIC
#define __PRIVATE
#define __PROTECTED
#define __LOCAL /*these variables are only used to avoid to many function parameters*/

/*
 * Theses three defines desribe the thread safety of an (pseudo) object.
 * Immutable: This object does not have a mutable state, so its thread safe.
 * Thread Safe: This object is thread safe when used with the give functions.
 * Not Thread Safe: This object is NOT thread safe, even when used with the
 *                  given functions.
 * Thread Confined: These objects are only used in one thread, so they do not need
 *                  to be guarded.
 */
#define __IMMUTABLE
#define __THREAD_SAFE
#define __NOT_THREAD_SAFE
#define __TCONFINED

#endif /*DOC_HELPS_H*/

