/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "kernel_user_interface.h"
#include "../prrt_socket.h"

static const char *DEBUG_NAME = "KUI_WRITE";

static int kui_write_write_proc(struct file *instanz, const char __user *userbuffer,
					unsigned long count, void *data);


/*From socket_manager.c*/
int sm_get_prrt_sock(Socketno_t socketno, struct prrt_socket **msock);

/*
 * Function to create a write file.
 */
int kui_create_write_file(Socketno_t socketno)
{
	char name[NAME_SIZE];
	struct proc_dir_entry *procentry;

	snprintf(name, NAME_SIZE, "prrt/%u/write", socketno);

	procentry = create_proc_entry(name, S_IWUGO, NULL);
	if (procentry) {
		procentry->read_proc  = NULL;
		procentry->write_proc = kui_write_write_proc;
		procentry->data = NULL;

		return 1;	/*SUCCESS*/
	}

	return 0;	/*FAILURE*/
}

/*
 * Function for deleting a procfs write file.
 */
void kui_delete_write_file(Socketno_t socketno)
{
	char name[NAME_SIZE];

	snprintf(name, NAME_SIZE, "prrt/%u/write", socketno);

	remove_proc_entry(name, NULL);
}

static int kui_write_write_proc(struct file *file, const char __user *userbuffer,
					unsigned long count, void *data)
{
	struct prrt_packet_container *pcont = 0;
	int not_copied = 0;
	int err = 0;
	struct prrt_socket *msock = 0;
	char* i_name = file->f_path.dentry->d_parent->d_iname;

	Socketno_t socketno = (Socketno_t)simple_strtoul(i_name, NULL, 10);

	PDEBUG(MSG_COMMONSEND, "%s: Write called on socket %d\n", DEBUG_NAME, socketno);

	if(sm_get_prrt_sock(socketno, &msock) == -ERESTARTSYS) {
		err = -ERESTARTSYS;
		goto error;
	}
	
	if(!msock) {
		/*ERROR: socket not found*/
		PERROR("%s: Socket not found\n", DEBUG_NAME);
		err = -2;
		goto error;
	}

	pcont = prrt_packet_alloc(&(msock->send_time_state));
	if(!pcont) {
		return -ENOMEM;
	}

	if(count > MAX_DATA_SIZE) {
		PERROR("%s: User tried to write too much data in one write\n", DEBUG_NAME);
		err = -ENOMEM;
		goto error;
	}

	not_copied = copy_from_user(&(pcont->pack.data.data), userbuffer, count);

	PDEBUG(MSG_KUI, "%s: Sending %lu bytes of data in one packet\n", DEBUG_NAME, count);
	//err = prrt_send(msock, pcont, count);
	if(err < 0) {
		PERROR("%s: Error on send(%d)\n", DEBUG_NAME,	err);
		goto error;
	}

	return count-not_copied;

error:
	prrt_packet_destroy(&pcont);
	PDEBUG(MSG_COMMONSEND, "%s: Returning error %d\n", DEBUG_NAME, err);
	return err;
}
