/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_LIST_H
#define PRRT_LIST_H

#include "../defines.h"
#include "../prrt_packet.h"
#include "../prrt_fbpacket.h"
#include "../stats/list_stats.h"

#include <linux/list.h>
#include <linux/slab.h>
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

/**
 * Structure that represents an entry in a list containing packets.
 */
struct packet_list_entry {
	struct list_head list;			/**< Used to connect entries (see kernel lists)*/
	struct prrt_packet_container *pcont;	/**< The struct that contains a packet*/
	u16 packetnumber;			/**< Sequence number for sorted list insertion*/
};

/**
* Initialise cache for packet_list_entry structure.
*/
int prrt_entry_cache_init(void);

/**
 * Uninitialise cache for packet_list_entry structure.
 */
void prrt_entry_cache_uninit(void);

/**
 * Allocate a packet_list_entry.
 */
int prrt_list_entry_alloc(struct packet_list_entry **p);

/**
* Free a packet_list_entry.
*/
void prrt_list_entry_free(struct packet_list_entry **p);

/**
 * Delete a packet from a list containing packet_list_entries.
 * \param iter iterator to the packet in the list.
 */
void prrt_list_delete_packet_from_list(struct list_head *iter);

/**
 * Delete all packets in a list containing packet_list_entries.
 * \param head the head of the list whose contents should be deleted.
 */
void prrt_list_delete_all_packets_in_list(struct list_head *head);

/**
 * Find position for a packet in a list ordered according to sequence numbers.
 * Insert the packet with prrt_list_insert_packet_behind_position.
 * \param head the head of the list to insert into.
 * \param packetNumber the number of the packet to insert.
 * \param position this will point to the position to insert the packet at.
 */
int prrt_list_find_position_for_sorted_insertion(struct list_head *head, u16 packetNumber,
						struct list_head **position);

/**
 * Insert packet at the position that was found for it.
 * The position is retrieved with a call to prrt_list_find_position_for_sorted_insertion.
 * \param pcont packet to insert.
 * \param position position to insert the packet at.
 */
int prrt_list_insert_packet_behind_position(struct prrt_packet_container *pcont,
					   struct list_head *position);

#endif

