/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef RECEIVER_LIST_H
#define RECEIVER_LIST_H

#include "../defines.h"
#include "../parameters/param_network.h"
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

/**
 * This structure represents an entry in the receiver list.
 */
struct receiver_list_entry {
	struct list_head 	list;
	u32			recv_addr;	/**<The ip address of this receiver.*/
	struct params_network	net_params;	/**<The network connection parameters for this
						  * receiver.*/
};

/**
 * This list is used to overview the receivers that joined a multicast session.
 * Receivers that send feedback are inserted in this list to save their connection parameters.
 * In this way we are able to choose the worst parameters for code choice.
 */
__THREAD_SAFE __PCLASS struct prrt_receiver_list {
	__PRIVATE
	struct list_head head;	/**<The head of the receiver list*/
	struct semaphore lock;	/**<This lock should synchronize access to the class.*/
	atomic_t count;		/**<A semaphore used to count the elements of the list.*/

	__PUBLIC
	struct params_network worstNetParams;	/**<This are the parameters used at the moment. They
						 *  are the worst of all receivers.*/
};

/**
 * Initialise the elements of the list.
 */
void prrtrlist_init(struct prrt_receiver_list *rlist);
int prrtrlist_reinit(struct prrt_receiver_list *rlist);

/**
 * Delete all entries in the list.
 */
void prrtrlist_free(struct prrt_receiver_list *rlist);

/**
 * Updates the set of worst parameters to the worst value of each parameter in the list.
 */
void prrtrlist_update_WorstNetParams(struct prrt_receiver_list *rlist);

/**
 * Checks for a receiver in the list and updates its t_replace.
 * If the receiver is not in the list it will be added to the list.
 * This will also update the worst parameters if necessary.
 */
void prrtrlist_update_ReplaceTime(struct prrt_receiver_list *rlist, u32 receiver, int sample_t_replace);

/**
 * Set the PLR for a given receiver.
 * If the receiver is not in the list it will be added to the list.
 * This will also update the worst parameters if necessary.
 */
void prrtrlist_setPLR(struct prrt_receiver_list *rlist, u32 receiver, int PLR);

/**
 * Set the worst replace time for testing purposes.
 */
void prrtrlist_setWorstTReplace(struct prrt_receiver_list *rlist, int t_replace);

void prrtrlist_print_stats(struct prrt_receiver_list *rlist);

#endif /*RECEIVER_LIST_H*/

