/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef _PARAM_LOCAL_H
#define	_PARAM_LOCAL_H

#include "../defines.h"
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif
#include <asm/atomic.h>

/**
 * This class contains parameters that are needed for the code computation but are not dependent
 * on any of the receivers.
 */
__THREAD_SAFE __PCLASS struct params_local {
	__PRIVATE
	struct semaphore lock;	/**<Lock to make the whole thing thread safe.*/

	/*This are not really network parameters rather constraints for them.*/
	atomic_t D_target;	/**<The given maximum target delay of a receiver in ms.*/
	atomic_t PLR_target;	/**< The PLR that has to be achieved for a receiver.*/
        atomic_t D_Fec;         /**< Maximum block coding delay.*/
        atomic_t BW_est;        /**< Estimated available bandwidth*/
        atomic_t BUF_fb;        /**< The buffer information from receiver application is sent to the sender as a feedback for flow control or scheduling packet sending. The unit can be various, specified by application.*/
        atomic_t Scheduled_RI;  /**< Redundancy bytes scheduled for sending*/
};

/**
 * Initialise the parameter values.
 */
void prrtlpar_init_locparams(struct params_local *params);
void prrtlpar_reinit_locparams(struct params_local *params);

/**
 * Set the maximum target delay of a receiver.
 */
void prrtlpar_set_D_target(struct params_local *params, int d_target);

/**
 * Retrieve the maximum target delay of a receiver.
 */
int prrtlpar_get_D_target(struct params_local *params);

/**
* Set the maximum target PLR of a receiver.
*/
void prrtlpar_set_PLR_target(struct params_local *params, int plr_target);

/**
* Retrieve the maximum target PLR of a receiver.
*/
int prrtlpar_get_PLR_target(struct params_local *params);

/**
 * Retrieve the maximum block Coding Delay.
 */
void prrtlpar_set_D_Fec(struct params_local *params, int d_fec);

/**
 * Retrieve the maximum block Coding Delay.
 */
int prrtlpar_get_D_Fec(struct params_local *params);

/**
 * Set the estimated available bandwidth.
 */
void prrtlpar_set_BW_est(struct params_local *params, int bw_est);

/**
 * Retrieve the estimated available bandwidth.
 */
int prrtlpar_get_BW_est(struct params_local *params);

/**
 * Set the buffer feedback information from receiver application.
 */
void prrtlpar_set_BUF_fb(struct params_local *params, int buf_fb);

/**
 * Retrieve the buffer feedback information from receiver application.
 */
int prrtlpar_get_BUF_fb(struct params_local *params);

/**
 * Set the estimated available bandwidth.
 */
void prrtlpar_update_Scheduled_RI(struct params_local *params, int RI_increment);

/**
 * Retrieve the estimated available bandwidth.
 */
int prrtlpar_get_Scheduled_RI(struct params_local *params);

#endif	/* _PARAM_LOCAL_H */

