/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <linux/net.h>
#include <linux/slab.h>
#include <linux/in.h>
#include <linux/syscalls.h>
#include <linux/types.h>
#include <linux/inet.h>
#include <linux/sched.h>
#include <linux/delay.h>

#include "defines.h"
#include "prrt_socket.h"
#include "prrt_relay.h"
#include "relay_packets.h"

#define DEBUG_NAME "PRRT_RELAY"

int prrt_relay(struct prrt_relay **mrelay, char *relayname)
{
  int i = 0;
	int err = 0;

	PDEBUG(MSG_COMMON, "%s: Creating new mrelay\n", DEBUG_NAME);

	/*TODO: Mem management*/
	*mrelay = kmalloc(sizeof(**mrelay), GFP_KERNEL);
	if (!(*mrelay)) {
		err = -ENOMEM;
		goto error;
	}

	(*mrelay)->relay_status = DOWN;

	/*initialise work queues to null*/
	(*mrelay)->relay_queue = 0;

	/*initialise variables for packet shipment*/
	INIT_DELAYED_WORK(&(*mrelay)->relayData_work, prrt_relay_packets);
  init_MUTEX(&(*mrelay)->socket_lock);
	init_MUTEX(&(*mrelay)->relayData_lock);

  for(i=0; i<RELAYNAMELENGTH; i++)
    (*mrelay)->relayname[i] = relayname[i];
  (*mrelay)->relayname[i] = '\0';

	return 0;

error:
	kfree(*mrelay);
	return err;
}

int prrt_start_relay(struct prrt_relay *mrelay)
{
	if (mrelay->relay_status == WORKING)
		return 0;

	if (mrelay->relay_queue == 0)
		mrelay->relay_queue = create_singlethread_workqueue("prrt_relay");

	mrelay->relay_status = WORKING;

	queue_delayed_work(mrelay->relay_queue, &mrelay->relayData_work, POLLDELAY);

  // Deactivate the read packet queue in prrt receiving socket, other it will read out about one half of incoming packets which are relayed.
  down(&mrelay->msock_in->poll_lock);
  mrelay->msock_in->poll_status = RELAY_WORKING;
  up(&mrelay->msock_in->poll_lock);

  return 0;
}

void prrt_shutdown_relay(struct prrt_relay *mrelay)
{
	down(&mrelay->relayData_lock);
	mrelay->relay_status = DOWN;
	PDEBUG(MSG_COMMON, "%s: Poll status set to %d\n", DEBUG_NAME, mrelay->relay_status);
	up(&mrelay->relayData_lock);

	//cancel_delayed_work(&mrelay->relayData_work);
	PDEBUG(MSG_COMMON, "%s: Flushing relay queue\n", DEBUG_NAME);
	if (mrelay->relay_queue != 0)
		flush_workqueue(mrelay->relay_queue);

	down(&mrelay->relayData_lock);

	if (mrelay->relay_queue != 0)
		destroy_workqueue(mrelay->relay_queue);

	up(&mrelay->relayData_lock);

	kfree(mrelay);
}
