/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_CLOCK_H
#define PRRT_CLOCK_H

#include "defines.h"

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#else
#include <linux/semaphore.h>
#endif

#define MAX_TIMESTAMP 4294967295ul
#define HALF_TIMESTAMP 2147483648ul

/**
 * This class encapsulates needed state for the time measurement of one socket instance.
 */
__THREAD_SAFE __PCLASS struct prrt_time_state {
	__PRIVATE
	atomic_t mean_deviation;  /**< The mean deviation of the local time to the reference time.*/
	atomic_t clock_skew;  /**< The clock skew between reference time and local time (in 0/00).*/
	atomic_t last_measurement; /**< Time of last measurement.*/ 
	atomic_t virtual_time; /**< Virtual clock (skew-compensated via reference time).*/ 
};

void prrttime_init(struct prrt_time_state *tstate);

/**
 * Compares the timestamps of two packets.
 * The assumption is that two timestamps will never lie more
 * than half of the maximal number range apart.
 * \param t1 The timestamp of the first packet.
 * \param t2 the timestamp of the second packet.
 * \return 1 if t1 > t2 | 0 if t1 = t2 | -1 if t1 < t2.
 */
int prrt_compare_timestamps(struct prrt_time_state *tstate, u32 t1, u32 t2);

/**
 * Build the difference between two timestamps.
 */
u32 prrt_diff_timestamps(struct prrt_time_state *tstate, u32 t1, u32 t2);

u32 prrt_get_mstime(struct prrt_time_state *tstate);

u32 prrt_get_ustime(struct prrt_time_state *tstate);

u32 prrt_get_prrttime(struct prrt_time_state *tstate);

void prrt_update_prrttime(struct prrt_time_state *tstate, u32 reference_time, u32 propagation_delay);
#endif /*PRRT_CLOCK_H*/
