/*
 * (c) 2010 Saarland University, Telecommunications Lab
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 +
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 +
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 +
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 +
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 +
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device,
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "defines.h"
#include "prrt_relay.h"

#define DEBUG_NAME "RELAY_PACKETS"

static int RelayStatusEquals(struct prrt_relay *mrelay, enum worker_thread_status_t status)
{
	int equals = 0;
	down(&mrelay->relayData_lock);
	equals = mrelay->relay_status == status;
	up(&mrelay->relayData_lock);
	return equals;
}

static int relay_packet(struct prrt_relay *mrelay)
{
	int err = 0;
	int dataBytes = 0;
	struct prrt_packet_container *pcont = 0;

        down(&mrelay->socket_lock);
        if(mrelay->msock_in == NULL){
            up(&mrelay->socket_lock);
            err = -ESHUTDOWN;
            PERROR("%s: Relaying packets failed, a socket was invalidated\n", DEBUG_NAME);
            return err;
        }

        PDEBUG(MSG_COMMONRECV, "%s: Before recv.\n", DEBUG_NAME);

        //PDEBUG(MSG_FINAL, "%s: Timestampe before receive %d", DEBUG_NAME, prrt_get_mstime());

	err = prrt_recv(mrelay->msock_in, &pcont);
        up(&mrelay->socket_lock);
	if (err < 0) {
		PERROR("%s: Relaying packets failed, could not receive packet, error no is %d\n", DEBUG_NAME, err);
		return err;
	}
        
        PDEBUG(MSG_COMMONRECV, "%s: After recv.\n", DEBUG_NAME);

        down(&mrelay->socket_lock);
        if(mrelay->msock_out == NULL){
            up(&mrelay->socket_lock);
            err = -ESHUTDOWN;
            PERROR("%s: Relaying packets failed, a socket was invalidated\n", DEBUG_NAME);
            return err;
        }

        PDEBUG(MSG_COMMONRECV, "%s: Before send.\n", DEBUG_NAME);

	dataBytes = err;

        prrt_packet_redundancy_to_data(pcont);

	err = prrt_send(mrelay->msock_out, pcont, dataBytes);
        up(&mrelay->socket_lock);
	if (err < 0) {
		PERROR("%s: Relaying packets failed, could not send packet, error no is %d\n", DEBUG_NAME, err);
		return err;
	}

        PDEBUG(MSG_COMMONRECV, "%s: After recv.\n", DEBUG_NAME);

	return 0;
}

static void handle_relaying(struct prrt_relay *mrelay)
{
	int err = 0;

	while (RelayStatusEquals(mrelay, WORKING)) {
		err = relay_packet(mrelay);
		if (err == -ENODATA || err == -EDESTADDRREQ || err == -ESHUTDOWN)
			break;

                down(&mrelay->relayData_lock);
		if(mrelay->relay_status == ERROR || mrelay->relay_status == DOWN) {
			up(&mrelay->relayData_lock);
			return;
		}
		up(&mrelay->relayData_lock);
	}

	PERROR("%s: Relaying packets failed, error no is %d\n", DEBUG_NAME, err);

	down(&mrelay->relayData_lock);
	mrelay->relay_status = DOWN;
	up(&mrelay->relayData_lock);
}

void prrt_relay_packets(struct work_struct *work)
{
	struct prrt_relay *mrelay = 0;
	struct delayed_work *dw = 0;
	enum worker_thread_status_t status = 0;

	dw = container_of(work, struct delayed_work, work);
	mrelay = container_of(dw, struct prrt_relay, relayData_work);

	// 	PDEBUG(MSG_COMMONRECV, "%s: Taking spinlock\n", DEBUG_NAME);
	down(&mrelay->relayData_lock);
	status = mrelay->relay_status;
	up(&mrelay->relayData_lock);

	PDEBUG(MSG_COMMONRECV, "%s: Relaying packets..\n", DEBUG_NAME);
	switch (status) {
	case WORKING:
        case RELAY_WORKING:
		handle_relaying(mrelay);
		break;
	case ERROR:
		/*This is an error*/
		//printk(MSG_TRACE "%s: Invalid trace..\n", DEBUG_NAME);
		//handle_sleeping(msock);
		break;
	case DOWN:
		//handle_down(msock);
		break;
	}

	PDEBUG(MSG_COMMONRECV, "%s: Relaying packets done\n", DEBUG_NAME);
}
