/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "send_data.h"
#include "defines.h"
#include "prrt_socket.h"
#include "prrt_time.h"

#define DEBUG_NAME "SEND_DATA"

static void handle_sending(struct prrt_socket *msock)
{
  int err = 0;

  for (;;) {

    PRINT_TRACE("SEND", "S", "waitfor_nextpack");

    /*this will be upped when the socket is closed*/
    while (prrtolist_wait_for_next_packet(&msock->out_packet_list) == -ETIME) {
      //prrtolist_try_to_code_next_block(&msock->out_packet_list, &msock->cod_params, &msock->loc_params_out);
      prrtolist_try_to_code_next_block(&msock->out_packet_list, &msock->cod_params);

      /*leave loop when shutting down socket*/
      down(&msock->out_lock);
      if (msock->out_status == DOWN) {
        up(&msock->out_lock);
        break;
      }
      up(&msock->out_lock);
    }

    /*leave loop when shutting down socket*/
    down(&msock->out_lock);
    if (msock->out_status == DOWN) {
      up(&msock->out_lock);
      break;
    }
    up(&msock->out_lock);

    err = prrtolist_send_next_packet(&msock->out_packet_list, msock->sock, msock->peer);
    if (err < 0) {
      PERROR("%s: Failed to send block, stop sending, socket is now down\n",
          DEBUG_NAME);
      down(&msock->out_lock);
      msock->out_status = DOWN;
      up(&msock->out_lock);
      break;
    }

//    /*send all the redundancy that is available for the 0th round*/
//    err = 0;
//    while (err >= 0) {
//      err = prrtolist_try_to_send_redundancy_of_next_coded_block(&msock->out_packet_list,
//          msock->sock, msock->peer, msock->send_tokens);
//      if(err >= 0){
//        msock->send_tokens -= (u32)err;
//        PDEBUG(MSG_FINAL, "%s: Parity bytes sent: %d, Tokens available: %u\n", DEBUG_NAME, err, msock->send_tokens);
//      }
//      //if(err == 0)
//      //  PDEBUG(MSG_SENDFBDEB, "%s: Sent redundancy of one block\n", DEBUG_NAME);
//    }
//    //PDEBUG(MSG_SENDFBDEB, "%s: Sending of pre redundancy done\n", DEBUG_NAME);

  }
  //up(&msock->out_lock);
  PRINT_TRACE("SEND", "S", "STOP");
}

void prrt_send_packet(struct work_struct *work)
{
  struct prrt_socket *msock = 0;
  struct delayed_work *dw = 0;
  enum worker_thread_status_t status = 0;

  dw = container_of(work, struct delayed_work, work);
  msock = container_of(dw, struct prrt_socket, out_work);

  //  PDEBUG(MSG_COMMONSEND, "%s: Taking spinlock 'out'\n", DEBUG_NAME);
  down(&msock->out_lock);
  status = msock->out_status;
  up(&msock->out_lock);

  PDEBUG(MSG_COMMONSEND, "%s: Sending..\n", DEBUG_NAME);
  switch (status) {
    case WORKING:
      handle_sending(msock);
      break;
    case RELAY_WORKING:
      /* should not happen or not supported yet.*/
      break;
    case ERROR:
      /*should not happen*/
      //handle_notsending(msock);
      break;
    case DOWN:
      break;
  }

  PDEBUG(MSG_COMMONSEND, "%s: Sending done\n", DEBUG_NAME);
}
