/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include "send_feedback.h"
#include "defines.h"
#include "prrt_socket.h"

#include <linux/sched.h>
#include "kernel_wrappers.h"

#define DEBUG_NAME "SEND_FEEDBACK"

#define CHAN_FB_INTERVAL 40

static int send_lossfeedback(struct prrt_socket *msock);

void prrt_send_feedback(struct work_struct *work)
{
	//TODO: replace by while loop
	struct prrt_socket *msock = 0;
	struct delayed_work *dw = 0;
	enum worker_thread_status_t status;

	dw = container_of(work, struct delayed_work, work);
	msock = container_of(dw, struct prrt_socket, sendfb_work);

	down(&msock->poll_lock);
	status = msock->poll_status;
	up(&msock->poll_lock);

	PDEBUG(MSG_SENDFBDEB, "%s: Poll status is %d\n", DEBUG_NAME, status);

	if(status == DOWN)
		return;

	send_lossfeedback(msock);

	PDEBUG(MSG_SENDFBDEB, "%s: Sending feedback done\n", DEBUG_NAME);

// 	prrtilist_check_data_packets(&msock->inc_packet_list);

// TODO: Random number generator for feedback suppression
//    u8 rnd = 0;
//    get_random_bytes(&rnd, 1);
//    return rnd * D_sup / 255 + 1;

	PDEBUG(MSG_SENDFBDEB, "%s: Queuing work\n", DEBUG_NAME);
	queue_delayed_work(msock->sendfb_queue, &msock->sendfb_work, FEEDBACKDELAY);
}

//static int send_chanfeedback(struct prrt_socket *msock)
//{
//	int err = 0;
//	int msglen = 0;
//	u32 lastPacketTs = 0;
//	u32 lastPacketRecvTs = 0;
//	u32 prrtTimeNow = 0;
//	struct prrt_chanfbpacket_container *mfbcont = 0;
//	int d_target = prrtlpar_get_D_target(&msock->loc_params_in);
//
//	PDEBUG(MSG_SENDFBDEB, "%s: Gathering feedback\n", DEBUG_NAME);
//
//	err = prrt_chanfbpacket_alloc(&mfbcont);
//	if(err < 0) {
//		PERROR("%s: Could not allocate channel feedback packet\n", DEBUG_NAME);
//		return err;
//	}
//
//	msglen = CHANFBPACKETSIZE;
//	mfbcont->payload_size = CHANFBPACKETSIZE;
//
//	PDEBUG(MSG_PARAM, "%s: Host address is %d\n", DEBUG_NAME, msock->host->sin_addr.s_addr);
//
//	/*set header information of the feedback packet*/
//	mfbcont->fb.header.packtype = (u8)packtype_chanfb;
//	//mfbcont->fb.header.packetno = msock->poll_pno_fb++;
//	mfbcont->fb.fbheader.receiver_addr = msock->host_addr.s_addr;
//
//	lastPacketTs = atomic_read(&msock->inc_packet_list.lastPacketTs);
//	lastPacketRecvTs = atomic_read(&msock->inc_packet_list.lastPacketRecvTs);
//	prrtTimeNow = prrt_get_prrttime();
//	mfbcont->fb.fbheader.rtt_probe = lastPacketTs + prrt_diff_timestamps(prrtTimeNow, lastPacketRecvTs);
//	//PDEBUG(MSG_FINAL, "%s: RTT probe = %d\n", DEBUG_NAME, mfbcont->fbheader.rtt_probe);
//
//	//mfbcont->fb.chanfb.plr = prrtnpar_get_PLR(&msock->inc_packet_list.net_params);
//	//mfbcont->fb.chanfb.cor = 0;
//	//mfbcont->fb.chanfb.res_plr = 0;
//
//	prrt_chanfbpacket_hton(mfbcont);
//	err = wrap_kernel_sendmsg(msock->sockfb, msock->peerfb, &(mfbcont->fb), msglen);
//
//	if(err < 0) {
//		PERROR("%s: Error on send (no %d)\n", DEBUG_NAME, err);
//	} else {
//		prrt_chanfbpacket_ntoh(mfbcont);
//		//msock->inc_packet_list.feedbacks_sent += mfbcont->header.payload_size;
//		msock->inc_packet_list.fbpackets_sent++;
//		PDEBUG(MSG_SENDFBDEB, "%s: Feedback packet sent\n", DEBUG_NAME);
//		PDEBUG(PRINT_RECVWORK, "%s: Feedback packet sent\n", DEBUG_NAME);
//	}
//
//	prrt_chanfbpacket_free(&mfbcont);
//	PDEBUG(MSG_SENDFBDEB, "%s: Feedback sent, queue empty\n", DEBUG_NAME);
//
//return 0;
//}

static int send_lossfeedback(struct prrt_socket *msock)
{
	int err = 0;
	u16 repetitions = 1;
	int msglen = 0;
	int i = 0;
	u32 lastPacketTs = 0;
	u32 lastPacketRecvTs = 0;
	u32 msTimeNow = prrt_get_mstime(&msock->receive_time_state);
	u32 lastFBSentTs = (u32)atomic_read(&msock->inc_packet_list.lastFBSentTs);
	struct prrt_fbpacket_container *mfbcont = 0;
	int d_target = prrtlpar_get_D_target(&msock->loc_params_in);
	int chanfb_sent = 0;

	if (!lastFBSentTs){
		PDEBUG(MSG_SENDFBDEB, "%s: Initializing lastFBSentTS\n", DEBUG_NAME);
		atomic_set(&msock->inc_packet_list.lastFBSentTs, prrt_get_mstime(&msock->receive_time_state));
	}

	chanfb_sent = (prrt_compare_timestamps(&msock->receive_time_state, msTimeNow, lastFBSentTs + CHAN_FB_INTERVAL) < 0);

	PDEBUG(MSG_SENDFBDEB, "%s: Gathering feedback\n", DEBUG_NAME);

	//TODO: d_target is obsolete
	while(!chanfb_sent || (prrtilist_gather_feedback_packet(&msock->inc_packet_list, &mfbcont, d_target, &repetitions) >= 0)){

		// If no loss feedback is due, we just send feedback on channel quality once
		if (!mfbcont){
			PDEBUG(MSG_SENDFBDEB, "%s: Preparing channel feedback at %u\n", DEBUG_NAME, prrt_get_prrttime(0));
			err = prrt_fbpacket_alloc(&mfbcont);
			if(err < 0) {
				PERROR("%s: Could not allocate channel feedback packet\n", DEBUG_NAME);
				return err;
			}

			chanfb_sent = 1;
		}

		msglen = mfbcont->payload_size + PRRT_HEADER_SIZE + PRRT_FB_HEADER_SIZE;

		PDEBUG(MSG_PARAM, "%s: Host address is %d\n", DEBUG_NAME, msock->host->sin_addr.s_addr);

		/*set header information of the feedback packet*/
		mfbcont->fb.header.packtype = (mfbcont->fb.header.packtype & ~0xF) | ((u8)packtype_fb & 0xF);
 		mfbcont->fb.header.index = msock->poll_pno_fb++;
		mfbcont->fb.fbheader.receiver_addr = msock->host_addr.s_addr;

		lastPacketTs = atomic_read(&msock->inc_packet_list.lastPacketTs);
		lastPacketRecvTs = atomic_read(&msock->inc_packet_list.lastPacketRecvTsLocal);
		//prrtTimeNow = prrt_get_prrttime();
		mfbcont->fb.fbheader.rtt_probe = lastPacketTs + prrt_diff_timestamps(&msock->receive_time_state, msTimeNow, lastPacketRecvTs);
		//PDEBUG(MSG_SENDFBDEB, "%s: RTT probe = %u, lastPacketTs = %u\n", DEBUG_NAME, mfbcont->fb.fbheader.rtt_probe, lastPacketTs);
		//PDEBUG(MSG_FINAL, "%s: RTT probe = %u, lastPacketTs = %u, msTimeNow = %u, lastPacketRecvTs = %u\n", DEBUG_NAME, mfbcont->fb.fbheader.rtt_probe, lastPacketTs, msTimeNow, lastPacketRecvTs);

		mfbcont->fb.fbheader.ftt = prrtnpar_get_FTT(&msock->inc_packet_list.net_params);
		prrtnpar_update_PLR(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.plr = prrtnpar_get_PLR(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.gap = (u16)prrtnpar_get_GapLength(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.ngap = (u16)prrtnpar_get_NumGaps(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.burst = (u16)prrtnpar_get_BurstLength(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.nburst = (u16)prrtnpar_get_NumBursts(&msock->inc_packet_list.net_params);
		mfbcont->fb.fbheader.bw_est = prrtlpar_get_BW_est(&msock->loc_params_in);
		mfbcont->fb.fbheader.buf_fb = prrtlpar_get_BUF_fb(&msock->loc_params_in);
		//PDEBUG(MSG_FINAL, "%s: BW estimate %d\n", DEBUG_NAME, mfbcont->fb.fbheader.bw_est);
		//PDEBUG(MSG_FINAL, "%s: BUF feedback %d\n", DEBUG_NAME, mfbcont->fb.fbheader.buf_fb);
// 		mfbcont->header.timestamp = 0;	/*TODO: this is not needed anymore*/
		//mfbcont->fbheader.PLR = prrtnpar_get_PLR(&msock->inc_packet_list.net_params);
		PDEBUG(MSG_SENDFBDEB, "%s: Gathering feedback ...\n", DEBUG_NAME);

		prrt_fbpacket_hton(mfbcont);

		// repeat feedback in order to achieve higher reliability
		for (i=0; i<repetitions && !(err < 0); i++){
			err = wrap_kernel_sendmsg(msock->sockfb, msock->peerfb, &(mfbcont->fb), msglen);
			//PDEBUG(MSG_FINAL, "%s: Sending FB, repetition %d\n", DEBUG_NAME, i);
		}

		atomic_set(&msock->inc_packet_list.lastFBSentTs, prrt_get_mstime(&msock->receive_time_state));

		if(err < 0) {
			PERROR("%s: Error on send (no %d)\n", DEBUG_NAME, err);
		} else {
			prrt_fbpacket_ntoh(mfbcont);
			msock->inc_packet_list.feedbacks_sent += mfbcont->block_count;
			msock->inc_packet_list.fbpackets_sent++;
			PDEBUG(MSG_SENDFBDEB, "%s: Feedback packet sent\n", DEBUG_NAME);
			PDEBUG(PRINT_RECVWORK, "%s: Feedback packet sent\n", DEBUG_NAME);
		}

		prrt_fbpacket_free(&mfbcont);
	}
	PDEBUG(MSG_SENDFBDEB, "%s: Feedback sent, queue empty\n", DEBUG_NAME);

	return 0;
}

