/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef LIST_STATS_H
#define LIST_STATS_H

#include "../prrt_packet.h"

/**
 * This structure holds several statistics for a list.
 */
struct prrt_list_stats {
	unsigned long av_list_data;	/**<Average # of data bytes in list.*/
	unsigned long av_list_packs;	/**<Average # of packets in list.*/

	unsigned long max_list_data;	/**<Maximum # of data bytes in list.*/
	unsigned long max_list_packs;	/**<Maximum # of packets in list.*/

	unsigned long min_list_data;	/**<Minimum # of data bytes in list.*/
	unsigned long min_list_packs;	/**<Minimum # of packets in list.*/

	unsigned long count_packs;	/**<This will count the elements in the list*/
	unsigned long count_data;	/**<This will count the data bytes in the list*/

	struct semaphore lock;		/**<Lock for this structure.*/
};

/**
 * Initialise a prrt_list_stats structure.
 */
void prrtstats_init_list_stats(struct prrt_list_stats *mlstats);
void prrtstats_reinit_list_stats(struct prrt_list_stats *mlstats);

/**
 * Update the list statistics when adding a packet.
 * \param pcont packet that is inserted in the list.
 */
void prrtstats_update_list_stats_add(struct prrt_list_stats *mlstats, struct prrt_packet_container *pcont);

/**
 * Update the list statistics when deleting a packet.
 * \param pcont packet that is deleted from the list.
 */
void prrtstats_update_list_stats_del(struct prrt_list_stats *mlstats, struct prrt_packet_container *pcont);

unsigned long prrtstats_get_av_list_data(struct prrt_list_stats *mlstats);

unsigned long prrtstats_get_av_list_packs(struct prrt_list_stats *mlstats);

unsigned long prrtstats_get_max_list_data(struct prrt_list_stats *mlstats);

unsigned long prrtstats_get_max_list_packs(struct prrt_list_stats *mlstats);

unsigned long prrtstats_get_min_list_data(struct prrt_list_stats *mlstats);

unsigned long prrtstats_get_min_list_packs(struct prrt_list_stats *mlstats);

void prrtstats_print_list_stats(struct prrt_list_stats *mlstats);


#endif /*LIST_STATS_H*/
