/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#include <stdio.h>
#include "proc_access.h"
#include "socket_list.h"
#include "prrt_sockets.h"

#define DEBUG 0
#undef DEBUG

inline void DEBUG_MSG(char *msg)
{
#ifdef DEBUG
	printf("%s", msg);
#endif
}


#define REG_BUFF_SIZE 20	/*size of the buffer to read the feedback
				 *of the register file*/
#define CONTROL_BUFF_SIZE 100	/*size of the buffer to read the feedback
				 *of the register file*/

#define PRRT_PATH "/proc/prrt/"
#define REGISTER_FILE "register"
#define CONTROL_FILE "control"
#define WRITE_FILE "write"
#define READ_FILE "read"

int prrt_socket()
{
	char rbuf[REG_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	char *register_file_path = PRRT_PATH REGISTER_FILE;

	for(i=0; i<REG_BUFF_SIZE; i++)
		rbuf[i] = '\0';
	
	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(register_file_path, "create", 6);
	if(ret < 0)
		return ret;

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(register_file_path, rbuf, REG_BUFF_SIZE);
	if(ret < 0)
		return ret;

	sscanf(rbuf, "%d", &ret);

	if(socket_list_add(ret) < 0) {
		prrt_close(ret);
		ret = -1;
	}

	return ret;
}

int prrt_bind(int sockfd, char *sockaddr, unsigned short sockport, char *peeraddr)
{
	char rbuf[CONTROL_BUFF_SIZE];
	char wbuf[CONTROL_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	int wlen = 0;
	char control_file_path[50];

	for(i=0; i<CONTROL_BUFF_SIZE; i++) {
		rbuf[i] = '\0';
		wbuf[i] = '\0';
	}

	sprintf(control_file_path, PRRT_PATH"%d/"CONTROL_FILE, sockfd);
	wlen = sprintf(wbuf, "bind %s %d %s", sockaddr, sockport, peeraddr);

	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(control_file_path, wbuf, wlen);
	if(ret < 0)
		return ret;

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(control_file_path, rbuf, CONTROL_BUFF_SIZE);
	if(ret < 0)
		return ret;

	sscanf(rbuf, "%d", &ret);

	return ret;
}

int prrt_listen(int sockfd, int backlog)
{
	char rbuf[CONTROL_BUFF_SIZE];
	char wbuf[CONTROL_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	int wlen = 0;
	char control_file_path[50];

	sprintf(control_file_path, PRRT_PATH"%d/"CONTROL_FILE, sockfd);
	wlen = sprintf(wbuf, "listen %d", backlog);

	for(i=0; i<REG_BUFF_SIZE; i++)
		rbuf[i] = '\0';

	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(control_file_path, wbuf, wlen);
	if(ret < 0)
		return ret;

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(control_file_path, rbuf, CONTROL_BUFF_SIZE);
	if(ret < 0)
		return ret;

	sscanf(rbuf, "%d", &ret);

	return ret;
}

int prrt_accept(int sockfd)
{
	char rbuf[CONTROL_BUFF_SIZE];
	char wbuf[CONTROL_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	int wlen = 0;
	char control_file_path[50];

	sprintf(control_file_path, PRRT_PATH"%d/"CONTROL_FILE, sockfd);
	wlen = sprintf(wbuf, "accept");

	for(i=0; i<CONTROL_BUFF_SIZE; i++)
		rbuf[i] = '\0';

	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(control_file_path, wbuf, wlen);
	if(ret < 0)
		return ret;

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(control_file_path, rbuf, CONTROL_BUFF_SIZE);
	if(ret < 0)
		return ret;

	sscanf(rbuf, "%d", &ret);
	
	return ret;
}

int prrt_connect(int sockfd, char* sockaddr, unsigned short sockport, char *hostaddr)
{
	char rbuf[CONTROL_BUFF_SIZE];
	char wbuf[CONTROL_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	int wlen = 0;
	char control_file_path[50];

	for(i=0; i<CONTROL_BUFF_SIZE; i++) {
		rbuf[i] = '\0';
		wbuf[i] = '\0';
	}

	sprintf(control_file_path, PRRT_PATH"%d/"CONTROL_FILE, sockfd);
	wlen = sprintf(wbuf, "connect %s %d %s", sockaddr, sockport, hostaddr);

	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(control_file_path, wbuf, wlen);
	if(ret < 0)
		return ret;

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(control_file_path, rbuf, CONTROL_BUFF_SIZE);
	if(ret < 0)
		return ret;

	sscanf(rbuf, "%d", &ret);
	
	return ret;
}

void prrt_close(int sockfd)
{
	char rbuf[REG_BUFF_SIZE];
	char wbuf[REG_BUFF_SIZE];
	int i = 0;
	int ret = 0;
	int wlen = 0;
	char *register_file_path = PRRT_PATH REGISTER_FILE;

	for(i=0; i<REG_BUFF_SIZE; i++)
		rbuf[i] = '\0';

	DEBUG_MSG("writing to procfile\n");
	wlen = sprintf(wbuf, "delete %d", sockfd);
	ret = proc_write(register_file_path, wbuf, wlen);

	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(register_file_path, rbuf, REG_BUFF_SIZE);
	socket_list_del(sockfd);
}

int prrt_send(int sockfd, char *data, int len, int flags)
{
	int ret = 0;
	char write_file_path[50];

	sprintf(write_file_path, PRRT_PATH"%d/"WRITE_FILE, sockfd);

	DEBUG_MSG("writing to procfile\n");
	ret = proc_write(write_file_path, data, len);

	return ret;
}

int prrt_recv(int sockfd, char *data, int len, int flags)
{
	int ret = 0;
	char read_file_path[50];

	sprintf(read_file_path, PRRT_PATH"%d/"READ_FILE, sockfd);
	
	DEBUG_MSG("reading feedback from procfile\n");
	ret = proc_read(read_file_path, data, len);

	if(ret > 0) {
		data[ret] = '\0';
	} else {
		ret = -1;
	}

	/*printf("feedback: %s\n", data);*/

	return ret;
}

int prrt_setsockopt(int sockfd, int level, int option_name, const char *option_value,
		int option_len)
{
  /* Do nothing, just solve warning. */
  sockfd = 0;
  level = 0;
  option_name = 0;
  option_value = 0;
  option_len = 0;
  return -1;
}

int set_param(int sockfd, enum Param_t param, void *value)
{
	int ret = 0;
	char write_file_path[50];
	char data[50];
	int temp = 0;
	int len = 0;

	switch(param) {
	case TESTPARA:
		sprintf(write_file_path, PRRT_PATH"%d/testpara", sockfd);
		temp = *((int *)value);
		len = sprintf(data, "%d", temp);
		break;
	default:
		return -1;
		break;
	}

	DEBUG_MSG("Writing new value to param\n");
	ret = proc_write(write_file_path, data, len);

	return ret;
}

int get_param(int sockfd, enum Param_t param, void *value)
{
	int ret = 0;
	char read_file_path[50];
	char data[50];
	int len = 50;
	int *temp = (int *)value;


	switch(param) {
	case TESTPARA:
		sprintf(read_file_path, PRRT_PATH"%d/testpara", sockfd);
		break;
	default:
		return -1;
		break;
	}
	

	DEBUG_MSG("reading param value from procfile\n");
	ret = proc_read(read_file_path, data, len);

	if(ret > 0) {
		data[ret] = '\0';
		switch(param) {
		case TESTPARA:
			sscanf(data, "%d", temp);
			break;
		default:
			return -1;
			break;
		}
	} else {
		ret = -1;
	}

	return ret;
}
