/* 
 * (c) 2010 Saarland University, Telecommunications Lab 
 * (prrt@nt.uni-saarland.de)
 *
 * The contents of this file are subject to the Mozilla Public License
 + Version 1.1 (the "License"); you may not use this file except in
 + compliance with the License. You may obtain a copy of the License at
 + http://www.mozilla.org/MPL/
 + 
 + Software distributed under the License is distributed on an "AS IS"
 + basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 + License for the specific language governing rights and limitations
 + under the License.
 + 
 + The Original Code is "Predictably Reliable Real-time Transport (PRRT)".
 + 
 + The Initial Developer of the Original Code is Saarland University.
 + All Rights Reserved.
 + 
 + Contributor(s): Jochen Gruen (Saarland University), Manuel Gorius
 + (Saarland University), Yongtao Shuai (Saarland University).
 + 
 + Disclaimer: This Alpha version of PRRT is not supported and stable yet.
 + Do not use it in a production environment: it may crash your device, 
 + so please make sure that you have a backup of your data before
 + installing PRRT.
 */

#ifndef PRRT_SOCKETS_H
#define PRRT_SOCKETS_H


/**
 * Type for the different parameters that can be set for the protocol.
 * This type needs to be the same in the module and the lib.
 */
enum Param_t {
	TESTPARA = 0	/*Testparameter*/
};

/**
 * Creates an prrt socket.
 * No arguments needed because prrt is limited to udplite.
 * \return -1 if an error occured, else the socket number
 */
int prrt_socket();

/**
 * Binds and prrt socket to an address and a port.
 * \param sockfd the number of the socket that should be bound.
 * \param sockaddr address for the socket as a string (e.g. "127.0.0.1")
 * \param sockport the port the socket should be bound to
 * \param peeraddr the address the socket should send feedback to
 * \return 0 on success, -1 if an error occurs
 */
int prrt_bind(int sockfd, char *sockaddr, unsigned short sockport, char *peeraddr);

/**
 * Let the socket listen for incoming connections.
 * \param sockfd the number of the socket that should listen.
 * \param backlog maximal number of connections that can be queued.
 * \return 0 on success, -1 if an error occurs.
 */
/* int prrt_listen(int sockfd, int backlog);*/

/**
 * Let the socket accept an incoming connections.
 * \param sockfd the number of the socket that should listen.
 * \return number of the new socket on success, -1 if an error occurs.
 */
/* int prrt_accept(int sockfd);*/

/**
 * Let the socket connect to a peer.
 * \param sockfd the number of the socket that should listen.
 * \param sockaddr address for the peer as a string (e.g. "127.0.0.1")
 * \param sockport the port of the peer.
 * \param hostaddr the address for the incoming feedback.
 * \return 0 on success, -1 if an error occurs.
 */
int prrt_connect(int sockfd, char* sockaddr, unsigned short sockport, char *hostaddr);

/**
 * Close a socket.
 * \param sockfd the number of the socket to close.
 */
void prrt_close(int sockfd);

/**
 * Send data over a socket.
 * \param sockfd the number of the socket to write to.
 * \param data the data that should be written to the socket.
 * \param len the number of bytes contained in data.
 * \param flags not implemented yet.
 * \return number of written bytes on success, -1 if an error occurs.
 */
int prrt_send(int sockfd, char *data, int len, int flags);

/**
 * Receive data from a socket.
 * \param sockfd the number of the socket to read from.
 * \param data the the buffer for the data that is read.
 * \param len the number of bytes data can hold.
 * \param flags not implemented yet.
 * \return number of bytes read on success, -1 if an error occurs.
 */
int prrt_recv(int sockfd, char *data, int len, int flags);

/**
 * Set an option for a socket.
 * \param sockfd the number of the socket to set the option for.
 * \param level protocol level of the option.
 * \param option_name type of option to set(see sys/socket.h).
 * \param option_value the value the option is set too
 * \param option_len the size of the value option_value points to.
 * \return 0 on success, -1 if an error occurs.
 */
int prrt_setsockopt(int sockfd, int level, int option_name, const char *option_value,
		int option_len);

/**
 * Set a parameter for the prrt socket.
 * \param sockfd the number of the socket to set the parameter.
 * \param param the parameter that should be set.
 * \param value the value the parameter should be assigned.
 * \return ?
 */
int set_param(int sockfd, enum Param_t param, void *value);

/**
 * Get a parameter from the prrt socket.
 * \param sockfd the number of the socket to get the parameter from.
 * \param param the parameter that should be retrieved.
 * \param value the value of the parameter.
 * \return ?
 */
int get_param(int sockfd, enum Param_t param, void *value);

#endif /*PRRT_SOCKETS_H*/
